#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "general.h"

#define NPARAMS 4
#define NFILTERS 30

int rflag = 0;  /* rows control frequency 1.0 */
int bflag = 0;  /* both rows and columns control frequency 1.0 separately */


/*     number   method   parameters
**
**      1       Ideal     band: .167 - .50 of frequencies
**      2       Butterworth: .167 - .50, 1,1 
**      3       Exponential: .167 - .50, 1,1 
**      4       Ideal     band: .25 - .75
**      5       Butterworth:       "    ,1,1
**      6       Exponential:       "
**      7       Ideal     band: .33 - .99
**      8       Butterworth:       "    ,1,1
**      9       Exponential:       "
*/
/** $BG[Ns(B "m" $B$O;HMQ$9$k%U%#%k%?%j%s%0J}K!$rFCDj$9$k!#(B
**/
int m[NFILTERS] = { 0,1,2,3,1,2,3,1,2,3 };

/** $BG[Ns(B "ff" $B$OCf?4<~GH?t(B
double ff[NFILTERS] = { 0.0, 0.33, 0.33, 0.33, 0.5, 0.5, 0.5, 0.67, 0.67, 0.67 };

/** "ps" $B$O$=$l$>$l$N<jK!$N%Q%i%a!<%?(B
double ps[NFILTERS][NPARAMS] = {
                {0},
                {0.167,0.0, 0.50,0.0},
                {0.167,1.0, 0.50,1.0},
                {0.167,1.0, 0.50,1.0},
                {0.250,0.0, 0.75,0.0},
                {0.250,1.0, 0.75,1.0},
                {0.250,1.0, 0.75,1.0},
                {0.333,0.0, 0.99,0.0},
                {0.333,1.0, 0.99,1.0},
                {0.333,1.0, 0.99,1.0},
		/*  la,   lb,  ha,  hb */
};


int method, nr2;
double ha, hb, la, lb, la2, ha2, distance, band_center;

static select_method(int nfilter) 
{
  if ( nfilter >= NFILTERS || nfilter < 0 ){
    perror("select_method(illegal filter number)");
    exit (EXIT_FAILURE);
  }
  if ( (method = m[nfilter]) == 0){
    perror("select_method(undefined filter)\n");
    exit (EXIT_FAILURE);
  }
  fprintf(stderr,"bandpass: applying filter no.%d\n", nfilter);
  
  band_center = ff[nfilter];
  la2 = la = ps[nfilter][0];
  la = fabs(band_center - la);
  lb = ps[nfilter][1];
  if (method == 2)
    lb += lb;
  ha2 = ha = ps[nfilter][2];
  ha = fabs(band_center - ha);
  hb = ps[nfilter][3];
  if (method == 2)
    hb += hb;
  return OK;
}

static double modulate(void) 
{
  double d, absd;
  
  d = distance - band_center;
  absd = fabs(d);
  
  switch (method) {
  case 1: /* ideal filter */
    if( distance >= la2 && distance <= ha2)
      return (1.0);
    else return (0.0);
  case 2: /* Butterworth filter */
    if( d < 0 )
      return(1.0/(1.0 + pow(absd/la,lb)));
    else
      return(1.0/(1.0+pow(absd/ha,hb)));
  case 3: /* Exponential filter */
    if ( d < 0 )
      return( exp(-pow(absd/la,lb)));
    else
      return( exp(-pow(absd/ha,hb)));
  default:
    perror("modulate(undefined filtering method)");
    exit (EXIT_FAILURE);
  }
}

int main(int argc, char *argv[])
{
/*
  header hd;
*/

  int nf, nc, nr, nc2, row, col, x, y, pf;
  int i, frame, argcc, nums;
  
  int  spectrum, nfilter; 
  long n_read;

  double  mink, invmink, powy, val, mval, scale;
  double  *distr, *distc, *k, d;
  float *ppic;
  char  *pic;

  if ((argv[argc-1][0]=='-') && (argv[argc-1][1]=='D'))
    argc--;
  
  spectrum = 1;
  mink = 2.0;
  nfilter = 1;

  for (argcc=1; argcc < argc; argcc++) {
    if ( argv[argcc][0] != '-') {
      perror("bandpass(bad argument)");
      exit (EXIT_FAILURE);
    }
    else switch (argv[argcc][1]) {
    case 'b':
      bflag++;
      break;
    case 'f':
      argcc++;
      nfilter = atoi(argv[argcc]);
      break;
    case 'h':
      usage();
      break;
    case 'p':
      nfilter = 0;
      m[0] = atoi(argv[++argcc]);
      ff[0] = atof(argv[++argcc]);
      for (i=0; i<NPARAMS; i++) ps[0][i] = atof(argv[++argcc]);
      break;
    case 'r':
      rflag++;
      break;
    case 's':
      break;
    default:
      fprintf(stderr,"unrecognized option\n");
      usage();
    }
  }

  if (mink == 0.0){
    perror("exponent must not be zero");
    exit (EXIT_FAILURE);
  }
  invmink = 1.0 / mink;
  
  if (rflag && bflag){
    perror("bandpass:only one of -r and -b may be specified");
    exit (EXIT_FAILURE);
  }

  /*
  if (spectrum) {
  */
    header_default(&hd);
    hd.nframe = nf = 1;
    hd.nrow = nr = 128;
    hd.ncolumn = nc = 128;
    hd.pix_format = pf = Float;
    hd.bit_per_pixel = 32;
  /*
  } else header_read(stdin, &hd);
  */

    nf = 1;  nr = 128;  nc = 128;  pf = Float;  nr2 = nr / 2;  nc2 = nc / 2;
  
  if ( (distr = (double *)calloc(nr2 + 1, sizeof(double))) == 0 ||
       (distc = (double *)calloc(nc2 + 1, sizeof(double))) == 0)
    perr("core allocation failure\n");
  k = distr;
  scale = (rflag || bflag) ? nr2 : nc2;

  scale = (rflag) ? nr2 : nc2;
  nums = ( pf == Float) ? 1 : 2;
/*
  if (!(pic = (float *)calloc(nr * nc * nums, sizeof(float)))){
  if (!(pic = (float *)malloc(nr * nc * nums * sizeof(float)))){
*/
  if ((pic = (char *)calloc(sizeof(float), nr * nc * nums)) == NULL){
    perror("bandpass: can't allocate frame storage");
    exit (EXIT_FAILURE);
  }
  
  select_method(nfilter);

  /* spectrum $B$,??$N$H$-$K$O(B ppic  $B$K2hA|$rFI$_9~$`$+$o$j$K(B
     $B$9$Y$F$N%T%/%;%k$K(B 1.0 $B$rBeF~$9$k(B */
  if (spectrum) {
      ppic = (float *)pic;
      for (i=0; i< nr*nc; i++)
	  *ppic++ = 1.0;
  } else {
      if (fread(pic,1,sizeof(float) * nr * nc * nums,stdin) < sizeof(float)*nc*nr*nums){
	  perror("read error");	  exit (EXIT_FAILURE);
      }
  }
  ppic = (float *)pic;
  for (row=0; row<nr; row++) {
      y = row;
/*
      if ( pf == Complex )
	  y = (y + nr2) % nr;
*/
      y = abs(y - nr2);
      powy = distr[y];
      for (col=0; col<nc; col++) {
	  x = col;
/*
	if (pf == Complex)
	  x = (x + nc2) % nc;
*/
	  x = abs(x - nc2);
	  distance = pow(powy + distc[x], invmink);
	  mval = modulate();
	  *ppic++ *= mval;
	  if (pf != Float)
	      *ppic++ *= mval;
      }
  }

  /* $B7k2L$N=PNO(B
  if (write(1, pic, nr * nc * nums * sizeof(float)) !=
      nr * nc * nums * sizeof(float)){
      perror("bandpass: write error");
      exit (EXIT_FAILURE);
  }
  */
  return OK;
}
