#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "matrix_util.h"
#include "fileio_util.h"
#include "fft.h"

void usage(char *prgname)
{
    fprintf(stderr, "%s - FFT 2 dimension\n", prgname);
    fprintf(stderr, "Usage: %s [-i] [datafile]\n",prgname);
    fprintf(stderr, "Option -i: inverse fft\n");
    fprintf(stderr, "       --help: show this message\n");
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    matinfo In;
    matrix  Re1, Im1;
    int argn=1, size = 0, i;
    char *filename;
    FLAG Inverse = False;

    while(argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0'){
        if (keymatch(argv[argn], "--help")) {
            usage(argv[0]);
        } else if (keymatch(argv[argn], "-h")) {
            usage(argv[0]);
        } else if (keymatch(argv[argn], "-i")) {
            Inverse = True;
        } else { usage(argv[0]); }
        argn++;
    }
    
    if ( argn != argc ) { 
        if ((filename=(char *)malloc(strlen(argv[argn])+1)) == NULL) {
            fprintf(stderr, "Could not allocate memory at fft2d - %s\n", strerror(errno)); 
            exit (EXIT_FAILURE);
        }
        strcpy(filename, argv[argn]);
        *(filename+strlen(argv[argn])) = '\0';
    } else { 
        if ((filename = (char *)malloc(2)) == NULL) {
            fprintf(stderr, "Could not allocate memory at fft2d - %s\n", strerror(errno));
            exit (EXIT_FAILURE);
        }
        filename[0]='-'; filename[1]='\0';
    }

    In = matinfo_read(filename);

    if (Inverse) { size = In->n >> 1; } 
    else         { size = In->n; }
    if ( size != In-> m ) {
        fprintf(stderr, "%s must be a square matrix as a input matrix\n",argv[0]);
        exit (EXIT_FAILURE);
    }

    Re1 = matinit(size, In->m);
    Im1 = matinit(size, In->m);
 
    for (i=1; i<=size; i++) {
        if (memcpy((void *)(Re1+offs(size,In->m,i,1)),
                   (void *)(In->mat+offs(In->n,In->m,i,1)),
                   In->m * sizeof(scalar)) == NULL) {
            fprintf(stderr, "Could not copy Re matrix\n");
        }
    }

    if (Inverse) { 
        for (i=size+1; i<=In->n; i++) {
            if (memcpy((void *)(Im1+offs(size,In->m,i-size,1)),
                       (void *)(In->mat+offs(In->n,In->m,i,1)),
                       In->m * sizeof(scalar)) == NULL) {
                fprintf(stderr, "Could not copy Re matrix\n");
            }
        }
    }
    
    fprintf(stdout, "### Progname: %s %s %s\n", argv[0], Inverse ? "-i":"", filename);
    fprint_signature(stdout);

    if (!Inverse) {
        if (!fft2d(In->n, Re1, Im1, 1))
            exit (EXIT_FAILURE);
        fwritemat(stdout, Re1, In->n, In->n, "### Real Part:\n", "%.6g ");
        fwritemat(stdout, Im1, In->n, In->n, "### Imaginary Part:\n", "%.6g ");
    } else {
        if (!fft2d(size, Re1, Im1, -1))
            exit (EXIT_FAILURE);
        fwritemat(stdout, Re1, size, size, "### Result of Inverse FFT:\n", "%.6g ");
    }
    return EXIT_SUCCESS;
}
