#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "general.h"
#include "fileio_util.h"

void fileio_util_perror (char *reason)
{
    extern int errno;
    char* e;

#ifndef A_STRERROR
    e = strerror(errno);
#else 
    e = sys_errlist[errno];
#endif A_STRERROR

  if ( reason != 0 && reason[0] != '\0' )
      fprintf (stderr, "%s - %s\n", reason, e);
  else
      fprintf (stderr, "%s\n", e);
}

/* File open/close that handles "-" as stdin and checks errors. */
FILE* fileio_util_openr (char *name) 
{
    FILE* f;

    if ( (name==NULL) || (strcmp(name,"-")==0) || (strlen(name)==0) ) { 
         f = stdin;
    } else {
         f = fopen (name, "r");
    }

    if ( f == NULL ) {
	fprintf(stderr, "Could not open file in fileio_util_openr()\nFile name is "); 
	fileio_util_perror( name );
	exit (EXIT_FAILURE);
    }
    return f;
}

FILE* fileio_util_openw (char *name) 
{
  FILE* f;

  if ( (name==NULL) || (strcmp(name,"-")==0) || strlen(name)==0 ) {
      f = stdout;
      return f;
  }
  f = fopen( name, "w" );
  if ( f == NULL ) {
    fileio_util_perror (name);
    exit (EXIT_FAILURE);
  }
  return f;
}

void fileio_util_close (FILE *f) 
{
  fflush( f );
  if ( ferror( f ) )
    fprintf(stderr, "a file read or write error occurred at some point\n");
  if ( (f == stdin) || (f == stdout) || ( f == stderr) )
      return;

  if ( fclose(f) != 0 ) {
      fileio_util_perror ("fclose failed");
      exit (EXIT_FAILURE);
  }
}
