/*
 *	gaussian_kernel : generate 2d Gaussian kernel
 */
#include "general.h"
#include "matrix_util.h"
#include "fileio_util.h"

/* gaussian kernel generator */
matrix gaussian_kernel (scalar lambda, int size)
{
    int	i, j, cent;
    scalar c, ss, w1, w2;
    matrix A;

    A = matinit(size,size);

    ss = lambda * lambda;
    c = 1.0 / (2.0 * PI * lambda);
    cent = size /2 + 1;

    for (i=1; i<=size; i++){
	for (j=1; j<=size; j++){
	    w1 = (double)((i-cent) * (i-cent));
	    w2 = (double)((j-cent) * (j-cent));
	  *(A+offs(size,size,i,j)) = c * exp( -0.5 * (w1 + w2) / lambda );
	}
    }
    return A;
}

#ifdef STANDALONE
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

long loadingtime;

int usage(char *prgname) {
    fprintf(stderr,"%s: generate a 2d gaussian kernel\n",prgname);
    fprintf(stderr,"Usage :\n");
    fprintf(stderr,"\t%s -e radius_of_error(s.d) [-t whole_size]\n",prgname);
    exit (EXIT_FAILURE);
}

int main (int argc, char **argv) 
{
    int argn=1, size=0;
    scalar env_size=0.0;
    matrix A;

    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
         if (keymatch(argv[argn], "--help")) usage(argv[0]);
         if (keymatch(argv[argn], "-h")) usage(argv[0]);
         if (keymatch(argv[argn],"-e")){
             ++argn;
#ifdef SCALAR_AS_FLOAT
             if (argn==argc||sscanf(argv[argn],"%g",&env_size)!=1)
#else
             if (argn==argc||sscanf(argv[argn],"%lg",&env_size)!=1)
#endif
								 usage(argv[0]);
             }
         if (keymatch(argv[argn],"-t")){
             ++argn;
         if (argn==argc||(sscanf(argv[argn],"%d",&size)!=1) )
						 usage(argv[0]);
	       }
         argn++;
    }

    if (env_size <= 0.0) {
        fprintf(stderr,"%s: Invalid parameter, negative(or 0) envelop size? %f\n",argv[0],env_size);
        usage(argv[0]);
        exit (EXIT_FAILURE);
    }
    if (size > 0 ) size = size;
    else size = (int)(env_size * 3.0 * 2.0 + 1.0);

    A = gaussian_kernel((double)env_size,size);

    fprintf(stdout, "### ProgramName: %s -e %g -t %d\n",argv[0],env_size,size);
    fprint_signature(stdout);

    fwritemat(stdout,A,size,size,"","%8.5f ");
    return OK;
}
#endif STANDALONE
