#include <stdlib.h>
#include <float.h>
#include "matrix_util.h"
#include "fileio_util.h"

double graymax = 255.0;

int usage(char *progname)
{
    fprintf(stderr,"%s - convert a matrix to pgm file\n", progname);
    fprintf(stderr,"Usage: %s [datafile]\n", progname);
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    char *filename;
    int        argn=1;
    matinfo Din;
    matrix Out;
    double max = - DBL_MAX, min = DBL_MAX;
    int i,j;
    
    if ((argc != 2) && (argc != 1)) { usage(argv[0]); } 

    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "--help")) usage(argv[0]);
        if (keymatch(argv[argn], "-h")) usage(argv[0]);
        ++argn;
    }
    
    if (argc != 1) {
        if ((filename = (char *)malloc(strlen(argv[argn])+1)) == NULL) {
            fprintf(stderr, "Could not allocate memory at main() in %s\n", argv[0]);
            exit (EXIT_FAILURE);
        }
        if (strcpy(filename,argv[argn]) == NULL) {
            fprintf(stderr, "Could not strcpy in %s\n", argv[0]);
            exit (EXIT_FAILURE);
        }
        filename[strlen(argv[argn])] = '\0';
    } else {
        if ((filename = (char *)malloc(2)) == NULL) {
            fprintf(stderr, "Could not allocate memory at main() in %s\n", argv[0]);
            exit (EXIT_FAILURE);
        }
        filename[0]='-'; filename[1]='\0';
    }
    fprintf(stdout, "P2\n");
    Din = matinfo_read(filename);
    if (Din->n == 0 || Din->m == 0) usage(argv[0]);
    fprintf(stdout, "%d %d\n%d\n",Din->m,Din->n,(int)graymax);

    fprintf(stdout, "### ProgramName: %s %s\n", argv[0], filename);
    fprint_signature(stdout);

    Out = matinit(Din->n, Din->m);
    for (i=1; i<=Din->n; i++) {
        for (j=1; j<=Din->m; j++) {
            if ( *(Din->mat+offs(Din->n,Din->m,i,j)) > max )
                max = *(Din->mat+offs(Din->n,Din->m,i,j));
            if ( *(Din->mat+offs(Din->n,Din->m,i,j)) < min )
                min = *(Din->mat+offs(Din->n,Din->m,i,j));
        }
    }
    if ( max - min < eps ) {
        fprintf(stderr, "Invalid range of matrix at mat2pgm()\n");
        exit (EXIT_FAILURE);
    }
    for (i=1; i<=Din->n; i++) {
        for (j=1; j<=Din->m; j++) {
            *(Out+offs(Din->n,Din->m,i,j)) 
                 = (*(Din->mat+offs(Din->n,Din->m,i,j))-min) / (max - min);
            *(Out+offs(Din->n,Din->m,i,j)) = ceil(graymax * *(Out+offs(Din->n,Din->m,i,j)));
        }
    }

    fwritemat(stdout, Out, Din->n, Din->m, "", "%8.0f " );
    return EXIT_SUCCESS;
}
