#include "matrix_util.h"
#include "fileio_util.h"

int mataffine(matrix O, /* original matrix */
              int On,     /* X Center of image  */
	      int Om,     /* Y Center of image  */ 
	      scalar zx,  /* Zoom factor of X axis */
	      scalar zy,  /* Zoom factor of Y axis */ 
	      scalar deg, /* Rotation factor */ 
	      scalar px,  /* Movement factor for X axis */ 
	      scalar py,  /* Movement factor for Y axis */ 
	      matrix D  /* target matrix */)
{
    int i,j, n, m, xfrom, xto, yfrom, yto;
    double xs,ys,d, r, c, s, x, y, u, v, p, q;
    
    r = deg2rad(deg);
    c = cos(r);
    s = sin(r);
    xs = (double)Om/2.0; 
    xfrom = -(int)floor(xs); xto = (int)ceil(xs);
    ys = (double)On/2.0; 
    yfrom = -(int)floor(ys); yto = (int)ceil(ys);
    
    for (i=yfrom; i < yto; i++) {
	for (j=xfrom; j< xto; j++) {
	    d = 0.0;
	    u = (double)j - px;      v = (double)i - py;
	    x = (u*c - v*s)/zx;      y = (u*s + v*c)/zy;
	    m = (int)floor(x);      n = (int)floor(y);
	    p = x - (double)m;      q = y - (double)n;
	    
#ifdef DEBUG2
	    fprintf(stderr,"i=%d,j=%d,x=%3.1f,y=%3.1f,n=%d,m=%d,p=%3.1f,q=%3.1f\n",
		    i+(int)ys, j+(int)xs,x,y,n,m,p,q);
#endif
	    if ((yfrom <= n) && (n < yto) && (xfrom <= m) && (m < xto))
		d= (1.0-q) * ((1.0-p) * *(O+offs(On,Om,n+0+(int)ys,m+0+(int)xs))
			      + p    * *(O+offs(On,Om,n+0+(int)ys,m+1+(int)xs)))
		    + q  * ((1.0-p) * *(O+offs(On,Om,n+1+(int)ys,m+0+(int)xs))
			    + p    * *(O+offs(On,Om,n+1+(int)ys,m+1+(int)xs)));
	    else
		d = 0.0;
	    *(D+offs(On,Om,i+(int)ys, j+(int)xs)) = d;
	}
    }
    return OK;
}

#ifdef STANDALONE
#include <stdlib.h>

int usage(char *progname)
{
    fprintf(stderr,"%s\n", progname);
    fprintf(stderr,"Usage %s zoomx zoomy rotate diffx diffy [datafile]\n", progname);
    fprintf(stderr," zoomx : magnification factor of x axis\n");
    fprintf(stderr," zoomy : magnification factor of y axis\n");
    fprintf(stderr," rotate : rotation factor (degree)\n");
    fprintf(stderr," diffx : offset of x\n");
    fprintf(stderr," diffy : offset of y\n");
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    char *prgname = argv[0], *filename;
    int 	n=0, m=0, argn=1;
    double zoomx, zoomy, rotate, diffx, diffy;
    matinfo Din;
    matrix Data, Out;
    
    if ((argc != 6) && (argc != 7)) { usage(prgname); } 

    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "-help")) 
        usage(prgname);
        ++argn;
    }
    
    if (sscanf(argv[1], "%d", &n) != 1) 	usage(prgname);
    if (sscanf(argv[2], "%d", &m) != 1)	usage(prgname);
    
    zoomx = atof(argv[1]);
    zoomy = atof(argv[2]);
    rotate = atof(argv[3]);
    diffx  = atof(argv[4]);
    diffy  = atof(argv[5]);

    if (argc != 6) {
        if ((filename = (char *)malloc(sizeof(argv[argn])+1)) == NULL) {
            fprintf(stderr, "Could not allocate memory in mataffine()\n");
            exit (EXIT_FAILURE);
        }
        if (strcpy(filename,argv[argn]) == NULL) {
            fprintf(stderr, "Could not strcpy in mataffine()\n");
            exit (EXIT_FAILURE);
        }
    } else {
        if ((filename = (char *)malloc(2)) == NULL) {
            fprintf(stderr, "Could not allocate memory in mataffine()\n");
            exit (EXIT_FAILURE);
        }
        filename[0]='-'; filename[1]='\0';
    }
    Din = matinfo_read(filename);
           
    n = Din->n; m = Din->m; Data = Din->mat;
    if (n == 0 || m == 0) usage(prgname);
    
    Out = matinit(n,m);
    mataffine(Data, n,m, zoomx, zoomy, rotate, diffx, diffy, Out);
    
    fprintf(stdout, "### ProgramName: %s %g %g %g %g %g %s\n",
            prgname, zoomx, zoomy, rotate, diffx, diffy, filename);
    fprint_signature(stdout);
    fprintf(stdout, "### Rows: %d\n",n);
    fprintf(stdout, "### Columns: %d\n",m);
    fprintf(stdout, "### ZoomFactorOfXAxis: %.6g\n",zoomx);
    fprintf(stdout, "### ZoomFactorOfYAxis: %.6g\n",zoomy);
    fprintf(stdout, "### RotationFactor(degree): %.6g\n",rotate);
    fprintf(stdout, "### ParallelMovementFactorOfXAxis: %.6g\n", diffx);
    fprintf(stdout, "### ParallelMovementFactorOfYAxis: %.6g\n", diffy);
    
    fwritemat(stdout, Out,n,m, "", "" );
    return EXIT_SUCCESS;
}
#endif STANDALONE
