/* Perspective transformation
 *  written by Shinichi ASAKAWA <asakawa@twcu.ac.jp>
 *  at 1992.04.11(Ver. 1.0)
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "general.h"
#include "matrix_util.h"
#include "fileio_util.h"

/*  n = l * m */
int product(double l[4][4], double m[4][4], double n[4][4])
{
    int i,j,k;
    double p;

    for (i=0;i<4;i++){
	for (j=0;j<4;j++){
	    p = 0.0;
	    for (k=0; k<4; k++) {
		p += l[i][k] * m[k][j];
	    }
	    n[i][j] = p;
	}
    }
    return OK;
}

/* Caluculate Parameters for persepective transformation */
int param_set(double k[9], int xsize, int ysize, double a, double b,
              double x0, double y0, double z0, double x, double y, double z, 
              double t, double s)
{
    double l[4][4],m[4][4],n[4][4], k1,k2,k3,k4,k5,k6,k7,k8,k9;
    double u,v,w;
    int xs = xsize/2, ys = ysize/2;
  
    u = x  * Pi / 180.0; v = y * Pi / 180.0; w = z * Pi / 180.0;
    l[0][0] = 1.0 / xs; l[0][1] = 0.0;      l[0][2] = 0.0;      l[0][3] = 0.0;
    l[1][0] = 0.0;      l[1][1] =-1.0 / ys; l[1][2] = 0.0;      l[1][3] = 0.0;
    l[2][0] = 0.0;      l[2][1] = 0.0;      l[2][2] = 1.0;      l[2][3] = 0.0;
    l[3][0] = 0.0;      l[3][1] = 0.0;      l[3][2] = 0.0;      l[3][3] = 1.0;
    
    m[0][0] = a;        m[0][1] = 0.0;      m[0][2] = 0.0;      m[0][3] = 0.0;
    m[1][0] = 0.0;      m[1][1] = b;        m[1][2] = 0.0;      m[1][3] = 0.0;
    m[2][0] = 0.0;      m[2][1] = 0.0;      m[2][2] = 1.0;      m[2][3] = 0.0;
    m[3][0] = 0.0;      m[3][1] = 0.0;      m[3][2] = 0.0;      m[3][3] = 1.0;
    product(l, m, n);
    
    l[0][0] = 1.0;      l[0][1] = 0.0;      l[0][2] = 0.0;      l[0][3] = 0.0;
    l[1][0] = 0.0;      l[1][1] = 1;        l[1][2] = 0.0;      l[1][3] = 0.0;
    l[2][0] = 0.0;      l[2][1] = 0.0;      l[2][2] = 1.0;      l[2][3] = 0.0;
    l[3][0] = x0;       l[3][1] = y0;       l[3][2] = z0;       l[3][3] = 1.0;
    product(n, l, m);
    
    n[0][0] =  cos(w);  n[0][1] = sin(w);   n[0][2] = 0.0;      n[0][3] = 0.0;
    n[1][0] = -sin(w);  n[1][1] = cos(w);   n[1][2] = 0.0;      n[1][3] = 0.0;
    n[2][0] = 0.0;      n[2][1] = 0.0;      n[2][2] = 1.0;      n[2][3] = 0.0;
    n[3][0] = 0.0;      n[3][1] = 0.0;      n[3][2] = 0.0;      n[3][3] = 1.0;
    product(m, n, l);
    
    m[0][0] = 1.0;      m[0][1] = 0.0;      m[0][2] = 0.0;      m[0][3] = 0.0;
    m[1][0] = 0.0;      m[1][1] =  cos(u);  m[1][2] = sin(u);   m[1][3] = 0.0;
    m[2][0] = 0.0;      m[2][1] = -sin(u);  m[2][2] = cos(u);   m[2][3] = 0.0;
    m[3][0] = 0.0;      m[3][1] = 0.0;      m[3][2] = 0.0;      m[3][3] = 1.0;
    product(l, m, n);
    
    l[0][0] = cos(v);   l[0][1] = 0.0;      l[0][2] = sin(v);   l[0][3] = 0.0;
    l[1][0] = 0.0;      l[1][1] = 1.0;      l[1][2] = 0.0;      l[1][3] = 0.0;
    l[2][0] = -sin(v);  l[2][1] = 0.0;      l[2][2] = cos(v);   l[2][3] = 0.0;
    l[3][0] = 0.0;      l[3][1] = 0.0;      l[3][2] = 0.0;      l[3][3] = 1.0;
    product(n, l, m);
    
    n[0][0] = 1.0;      n[0][1] = 0.0;      n[0][2] = 0.0;      n[0][3] = 0.0;
    n[1][0] = 0.0;      n[1][1] = 1.0;      n[1][2] = 0.0;      n[1][3] = 0.0;
    n[2][0] = 0.0;      n[2][1] = 0.0;      n[2][2] = -1.0;     n[2][3] = 0.0;
    n[3][0] = 0.0;      n[3][1] = 0.0;      n[3][2] = t;        n[3][3] = 1.0;
    product(m, n, l);
    
    m[0][0] = 1.0;      m[0][1] = 0.0;      m[0][2] = 0.0;      m[0][3] = 0.0;
    m[1][0] = 0.0;      m[1][1] = 1.0;      m[1][2] = 0.0;      m[1][3] = 0.0;
    m[2][0] = 0.0;      m[2][1] = 0.0;      m[2][2] = 1.0 / s;  m[2][3] = 1/s;
    m[3][0] = 0.0;      m[3][1] = 0.0;      m[3][2] = -1.0;     m[3][3] = 0.0;
    product(l, m, n);
    
    l[0][0] = xs;       l[0][1] = 0.0;      l[0][2] = 0.0;      l[0][3] = 0.0;
    l[1][0] = 0.0;      l[1][1] = -xs;      l[1][2] = 0.0;      l[1][3] = 0.0;
    l[2][0] = 0.0;      l[2][1] = 0.0;      l[2][2] = 1.0;      l[2][3] = 0.0;
    l[3][0] = 0.0;      l[3][1] = 0.0;      l[3][2] = 0.0;      l[3][3] = 1.0;
    product(n, l, m);
    
    k1 = m[0][3]; k2 = m[1][3]; k3 = m[3][3];
    k4 = m[0][0]; k5 = m[1][0]; k6 = m[3][0];
    k7 = m[0][1]; k8 = m[1][1]; k9 = m[3][1];
    k[0] = k7 * k2 - k8 * k1;
    k[1] = k5 * k1 - k4 * k2;
    k[2] = k4 * k8 - k7 * k5;
    k[3] = k8 * k3 - k9 * k2;
    k[4] = k6 * k2 - k5 * k3;
    k[5] = k5 * k9 - k8 * k6;
    k[6] = k9 * k1 - k7 * k3;
    k[7] = k4 * k3 - k6 * k1;
    k[8] = k7 * k6 - k4 * k9;
    return OK;
}

void usage(void)
{
    fprintf(stderr, "perspect: Perspective Transformation by linear insertion\n");
    fprintf(stderr, "\tProgramed by S.Asakawa 1993/04/05\n");
    fprintf(stderr, "Usage:\n");
    fprintf(stderr, "perspect zoomx zoomy px py pz rotx roty rotz view scr < image > image\n");
    fprintf(stderr, "Default settings:\n");
    fprintf(stderr, "zoomx=1.0\nzoomy=1.0\npx=0\npy=0\npz=0\nrotx=0.0\nroty=0\nrotz=0\nview=1.0 scr=1.0 < image > image\n");
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int d,i,j, xs,ys,m,n;
    double ax=1.0, ay=1.0, px=0.0, py=0.0, pz=0.0, rx=0.0, ry=0.0, rz=0.0;
    double view=1.0, screen=1.0;
    double k[9], x,y,w, p,q;
    /*    double r; */

    matinfo Dinfo;
    matrix Data, Out;
    int matn, matm;

    if (argc != 11) usage();
  
    ax = atof(argv[1]);    ay = atof(argv[2]);
    px = atof(argv[3]);    py = atof(argv[4]);    pz = atof(argv[5]);
    rx = atof(argv[6]);    ry = atof(argv[7]);    rz = atof(argv[8]);
    view  = atof(argv[9]);    screen  = atof(argv[10]);

    Dinfo = matinfo_read("-");
    Data = Dinfo->mat; matn = Dinfo->n; matm = Dinfo->m;
    Out = matinit(matn,matm);

    param_set(k, matm, matn, ax,ay,px,py,pz,rx,ry,rz,view,screen);
    xs = matm / 2;
    ys = matn / 2;
    
    for (i = -ys; i < +ys; i++) {
        for (j = -xs; j< +xs; j++) {
            w = k[0] * j + k[1] * i + k[2];
            x = k[3] * j + k[4] * i + k[5];
            y = k[6] * j + k[7] * i + k[8];

            x = x / w; 
            y = y / w;

            if(y > 0) m = y; else m = y - 1;
            if(y > 0) n = x; else n = x - 1;
            q = y - m;
            p = x - n;

            if ((-ys <= m) && (m < ys) && (-xs<=n) && (n< xs)) {
		/*
                d = (1.0-q) * ((1.0-p) * *(dp+(m +ys)  * matm + n + xs)
                      +             p  * *(dp+(m +ys)  * matm + n + xs + 1))
                      +  q  * ((1.0-p) * *(dp+(m+1+ys) * matm + n + xs)
                      +             p  * *(dp+(m+1+ys) * matm + n + xs + 1));
		*/
                d = (1.0-q) 
		    * ((1.0-p) * *(Data+offs(matn,matm,m+ys,n+xs))
		       +    p  * *(Data+offs(matn,matm,m+ys,n+xs+1))) +  
		    q  
		    * ((1.0-p) * *(Data+offs(matn,matm,m+ys+1,n+xs))
		       +    p  * *(Data+offs(matn,matm,m+ys+1,n+xs+1)));
            } else {
                d = 0.0;
            }
            /*
            if(d < 0)   d = 0;
            if(d > 255) d = 255;
            */
	    /*
            *(op + (i+ys) * matm + j + xs) = d;
	    */
	    *(Out+offs(matn,matm,i+ys,j+xs)) = d;
        }
    }
    fprintf(stdout, "### ProgramName: %s\n", argv[0]);
    fprint_signature(stdout);
    fwritemat(stdout,Out,matn,matm,"","");
    return OK;
}
