/* The definitions of some structures, constants, and functions for matrix_utils

   Copyright (c) 1995, 1996, 1997, 1999 Shinichi ASAKAWA <asakawa@twcu.ac.jp>
 
   Disclaimer:  No guarantees of performance accompany this software,
   nor is any responsibility assumed on the author.  */
#ifndef _MATRIX_UTIL_H_
#define _MATRIX_UTIL_H_

#include <stdio.h>
#include <math.h>
#include "general.h"

#ifdef scalar
#undef scalar
#endif
/* Definition of scalar. If you want to use float format as a scalar value,
   you must delete comment below, and comment out the definition of double  */
/*
#define scalar float
#define SCALAR_AS_FLOAT
#ifdef SCALAR_AS_DOUBLE
#undef SCALAR_AS_DOUBLE
#endif
*/
#define scalar double
#define SCALAR_AS_DOUBLE
#ifdef SCALAR_AS_FLOAT
#undef SCALAR_AS_FLOAT
#endif

typedef scalar *vector, *matrix;

#include <float.h>
#ifdef eps
#undef eps
#endif
#ifdef SCALAR_AS_DOUBLE
#define eps DBL_EPSILON
#else
#define eps FLT_EPSILON
#endif

#define	sizedef(n,m)	((n)*(m))
#define	offs(n,m,i,j)	((m)*((i)-1)+((j)-1))

/* The definition of SEPARATOR_CHARS, which are required in freadmat() 
   and matinfo_read() */
#ifdef SEPARATOR_CHARS
#undef SEPARATOR_CHARS
#endif
#define SEPARATOR_CHARS ", \t"

typedef struct MAT_info {
    int n;
    int m;
    matrix mat;
    char *name;
} *matinfo;

/* ------------------------------------------------------------
 * The body of functions below are described in matrix_util.c
 * ------------------------------------------------------------- */
matrix matinit(int, int);
int *intmatinit(int, int);
vector vectinit(int n);
double vectinnerprd(vector a, vector b, int n);
int  vectouterprd(vector a, vector b, int n, matrix Out);
matrix vectOuterprd(vector a, vector b, int n);
int vectnormal(vector x, int n); /* normalization of a vector whose length is 1.0 */

/* ---------------------------------
 * in matgen.c
 * --------------------------------- */
int matfill(matrix A, int n, int m, double filler);
matrix matFill(int n, int m, double filler);

/* ---------------------------------
 * in matI.c
 * --------------------------------- */
matrix matI(int n) ;

/* ---------------------------------
 * in matdiag.c
 * --------------------------------- */
int matdiag(matrix A, int n, matrix Ret); 
vector matDiag(matrix A, int n);

/* ---------------------------------
 * in mattrace.c
 * --------------------------------- */
int mattrace(matrix A, int n, double *trace);
scalar matTrace(matrix A, int n);

/* ---------------------------------
 * in matadd.c
 * --------------------------------- */
int matadd(matrix, int, int, matrix, int, int, matrix Ret);  /* Ret = A + B  */
int matADD(matrix, int, int, matrix, int, int);              /* A += B       */
matrix matAdd(matrix, int, int, matrix, int, int);           /* Return A + B */

/* ---------------------------------
 * in matsub.c
 * --------------------------------- */
int matsub(matrix, int, int, matrix, int, int, matrix Ret);  /* Ret = A + B  */
int matSUB(matrix, int, int, matrix, int, int);              /* A += B       */
matrix matSub(matrix, int, int, matrix, int, int);           /* Return A + B */

/* ---------------------------------
 * in matprd.c
 * --------------------------------- */
int matprd(matrix A, int an,int am , matrix B, int bn, int bm, matrix Ret); /* Ret=A*B */
int matPRD(matrix, int,int, matrix, int, int);               /* A *= B */
matrix matPrd(matrix, int,int, matrix, int, int);            /* Retrun A * B */

/* ---------------------------------
 * in matcpy.c
 * --------------------------------- */
int matcpy(matrix From, int an, int am, matrix To);          /* copy A to B */
matrix matCpy(matrix A, int an, int am);                /* return same contents of A */

/* ---------------------------------
 * in matarith.c
 * --------------------------------- */
int matarith(matrix A, int n, int m, double a, double b, matrix Ret); 
int matARITH(matrix A, int n, int m, double a, double b);
matrix matArith(matrix A, int n, int m, double a, double b);

/* ---------------------------------
 * in matt.c (transposed matrix)
 * --------------------------------- */
int matt(matrix A, int an, int am, matrix Ret);
matrix matT(matrix, int, int);

/* ---------------------------------
 * in matsq.c
 * --------------------------------- */
int matsq(matrix A, int n, int m, matrix Ret);
int matSQ(matrix A, int n, int m);
matrix matSq (matrix A, int n, int m);
int matsqinv(matrix A, int n, int m, matrix Ret);
int matSQINV(matrix A, int n, int m);
matrix matSqinv(matrix A, int n, int m);
int matsqinvmatt(matrix A, int n, int m, matrix Ret);
int matSQINVMATT(matrix A, int n, int m);
matrix matSqinvmatt(matrix A, int n, int m);

/* ---------------------------------
 * in gram_schmidt.c
 * --------------------------------- */
matrix Gram_Schmidt (matrix A, int n, int m);
int GRAM_SCHMIDT (matrix A, int n, int m);
int gram_schmidt (matrix A, int n, int m, matrix Ret);

/* ---------------------------------
 * in matnormalize.c
 * --------------------------------- */
matrix matNormalize (matrix A, int n, int m);
int matNORMALIZE (matrix A, int n, int m);
int matnormalize (matrix A, int n, int m, matrix Ret);

/* ---------------------------------
 * in matstandard.c
 * --------------------------------- */
matrix matStandard (matrix A, int n, int m);
int matSTANDARD (matrix A, int n, int m);
int matstandard (matrix A, int n, int m, matrix Ret);

/* ---------------------------------
 * in writemat.c
 * --------------------------------- */
int vectwrite(FILE *fp, matrix w, int size , unsigned char *mes);
int writemat(matrix w,           /* matrix to be written to FILE */
             int row,            /* size of row for matrix w     */
             int col,            /* size of column for matix w   */
             unsigned char *mes, /* header messages for output   */
	           unsigned char *fmt  /* format to be written         */ ) ;
int fwritemat(FILE *fp, matrix w, int row, int col, unsigned char *mes, unsigned char *fmt);

/* ---------------------------------
 * in matrix_fileio.c
 * --------------------------------- */
int freadmat(FILE *, matrix, int, int);
matrix matreadfile(char *filename, int n, int m);
int vectread(FILE *fp, matrix w, int size);

/* ---------------------------------
 *   described in matrix_inforead.c 
 * --------------------------------- */
matinfo matinfo_read(char *filename);

/* ---------------------------------
 * described in mattrim.c
 * --------------------------------- */
int mattrim(matrix O,   /* An origninal matrix to be trimmed */
	          int n,      /* rows of original matrix           */
	          int m,      /* columns of original matrix        */
	          int n1,     /* upper bound of trimmed matrix     */
	          int m1,     /* left bound of trimmed matrix      */
	          int n2,     /* lower boud                        */
	          int m2,     /* right boud                        */
	          matrix Ret  /* trimmed matrix                    */);
matrix matTrim(matrix O, int n, int m, int n1, int m1, int n2, int m2);

/* ---------------------------------
 * in plot2d.c
 * --------------------------------- */
int plot2d(FILE *fp, matrix x, int n, int m, char *header);

/* ---------------------------------
 * in matinv.c
 * --------------------------------- */
int matinv(matrix A, int n, scalar *det, matrix ret);
matrix matInv(matrix A, int n, scalar *det);
scalar determinant(matrix A, int m);

/* ---------------------------------
 * in bstat.c
 * --------------------------------- */
matrix bstat_min (matrix A, int n, int m);
matrix bstat_max (matrix A, int n, int m);
matrix bstat_range (matrix A, int n, int m);
matrix bstat_square (matrix A, int n, int m);
matrix bstat_sum (matrix A, int n, int m);
matrix bstat_mean (matrix A, int n, int m);
matrix bstat_variance (matrix A, int n, int m);
matrix bstat_sd (matrix A, int n, int m);
matrix bstat_skew (matrix A, int n, int m, vector mean, vector sd);
matrix bstat_kurtosis (matrix A, int n, int m, vector mean, vector sd);

/* ---------------------------------
 * in matcorr.c
 * --------------------------------- */
int matcorr (matrix A, int n, int m, matrix Corr);
matrix matCorr (matrix A, int n, int m);
int matcorrinv(matrix X, int n, int m, scalar *det, matrix corrinv);
matrix matCorrinv(matrix X, int n, int m, scalar *det);
int matubcov(matrix X, int n, int m, matrix Ret);
matrix matUbcov(matrix X, int n, int m);
int matcov (matrix X, int n, int m, matrix Cov);
matrix matCov (matrix X, int n, int m);
int matubcovinv (matrix X, int n, int m, scalar *det, matrix Covinv);
matrix matUbcovinv (matrix X, int n, int m, scalar *det);
int matcovinv(matrix X, int n, int m, scalar *det, matrix Covinv);
matrix matCovinv(matrix X, int n, int m, scalar *det);

/* ---------------------------------
 * in eigen_pm.c
 * --------------------------------- */
int eigen_pm(matrix a0, vector evv, matrix vv, int n, int p);

/* ---------------------------------
 * in matmask.c
 * --------------------------------- */
int matmask(matrix In, int tn, int tm, matrix Mask, int mn, int mm, matrix Ret);
int matMASK(matrix In, int tn, int tm, matrix Mask, int mn, int mm);
matrix matMask(matrix In, int tn, int tm, matrix Mask, int mn, int mm);

/* ---------------------------------
 * in matheviside.c
 * --------------------------------- */
int matHEVISIDE(matrix A, int n, int m, scalar thres);
matrix matHeviside(matrix A, int n, int m, scalar thres);
int matheviside(matrix A, int n, int m, scalar thres, matrix Ret);

/* ---------------------------------
 * in matabsdiff.c
 * --------------------------------- */
scalar dmatabs(matrix A, int an, int am, matrix B, int bn, int bm);
/*--- return Ret = |A - B| ---*/
int matabsdiff(matrix A, int an, int am, matrix B, int bn, int bm, matrix Ret);
/*--- return A -= B ---*/
int matABSDIFF(matrix A, int an, int am, matrix B, int bn, int bm);
matrix matAbsdiff(matrix A, int an, int am, matrix B, int bn, int bm);

/* not check  yet */
int matclr(matrix, int, int);
int MatrixRandomize(matrix, int, int, double);

#endif _MATRIX_UTIL_H_
