/*
 * @(#)matabsdiff.c - matrix operate function 1.02 (Shinichi Asakawa) 06-Mar.-2001  
 */
#include <stdlib.h>
#include "matrix_util.h"
#include "fileio_util.h"

scalar dmatabs(matrix A, int an, int am, matrix B, int bn, int bm)
{
  int	i,j;
  scalar ret=0.0;

  if ((an != bn) || (am != bm)) {
    fprintf(stderr,"Illegal size of matrix. Cannot calculate in dmatabs()\n");
    fprintf(stderr,"A(n*m):n=%d\tm=%d\n", an, am);
    fprintf(stderr,"B(n*m):n=%d\tm=%d\n", bn, bm);
    exit (EXIT_FAILURE);
  }

  if (A == NULL) {
      fprintf(stderr, "Matrix A is NULL at dmatabsdiff()\n");
      exit (EXIT_FAILURE);
  }
  if (B == NULL) {
      fprintf(stderr, "Matrix B is NULL at matabsdiff()\n");
      exit (EXIT_FAILURE);
  }

  for (i=1; i<=an; i++){
    for(j=1; j<=am; j++){
      ret += fabs(*(A+offs(an,bm,i,j)) - *(B+offs(bn,bm,i,j)));
    }
  }
  return ret;
}

/*--- return Ret = |A - B| ---*/
int matabsdiff(matrix A, int an, int am, 
           matrix B, int bn, int bm, 
           matrix Ret)
{
  int	i,j;

  if ((an != bn) || (am != bm)) {
    fprintf(stderr,"Illegal size of matrix. Cannot calculate in matsub()\n");
    fprintf(stderr,"A(n*m):n=%d\tm=%d\n", an, am);
    fprintf(stderr,"B(n*m):n=%d\tm=%d\n", bn, bm);
    exit (EXIT_FAILURE);
  }

  if (A == NULL) {
      fprintf(stderr, "Matrix A is NULL at matabsdiff()\n");
      exit (EXIT_FAILURE);
  }
  if (B == NULL) {
      fprintf(stderr, "Matrix B is NULL at matabsdiff()\n");
      exit (EXIT_FAILURE);
  }
  if (Ret == NULL) {
      fprintf(stderr, "Matrix Ret is NULL at matabsdiff()\n");
      exit (EXIT_FAILURE);
  }

  for (i=1; i<=an; i++){
    for(j=1; j<=am; j++){
      *(Ret+offs(an,am,i,j)) += fabs(*(A+offs(an,bm,i,j)) - *(B+offs(bn,bm,i,j)));
    }
  }
  return OK;
}

/*--- return A -= B ---*/
int matABSDIFF(matrix A, int an, int am, 
           matrix B, int bn, int bm)
{
  int	i,j;

  if ((an != bn) || (am != bm)) {
    fprintf(stderr,"Illegal size of matrix. Cannot calculate at matADD()\n");
    fprintf(stderr,"A(n*m):n=%d\tm=%d\n", an, am);
    fprintf(stderr,"B(n*m):n=%d\tm=%d\n", bn, bm);
    exit (EXIT_FAILURE);
  }
  if (A == NULL) {
      fprintf(stderr, "Matrix A is NULL at matABSDIFF()\n");
      exit (EXIT_FAILURE);
  }
  if (B == NULL) {
      fprintf(stderr, "Matrix B is NULL at matABSDIFF()\n");
      exit (EXIT_FAILURE);
  }

  for (i=1; i<=an; i++){
    for(j=1; j<=am; j++){
      *(A+offs(an,bm,i,j)) = fabs(*(A+offs(an,am,i,j)) - *(B+offs(bn,bm,i,j)));
    }
  }
  return OK;
}

matrix matAbsdiff(matrix A, int an, int am, 
              matrix B, int bn, int bm)
{
  int	i,j;
  matrix Ret;

  if ((an != bn) || (am != bm)) {
    fprintf(stderr,"Illegal size of matrix. Cannot calculate at matSub()\n");
    fprintf(stderr,"A(n*m):n=%d\tm=%d\n", an, am);
    fprintf(stderr,"B(n*m):n=%d\tm=%d\n", bn, bm);
    exit (EXIT_FAILURE);
  }

  if (A == NULL) {
      fprintf(stderr, "Matrix A is NULL at matAbsdiff()\n");
      exit (EXIT_FAILURE);
  }
  if (B == NULL) {
      fprintf(stderr, "Matrix B is NULL at matAbsdiff()\n");
      exit (EXIT_FAILURE);
  }

  if ((Ret = matinit(an, bm)) == NULL) {
      fileio_util_perror("Ret=matinit() failed at matAbsdiff()");
      exit (EXIT_FAILURE);
  }

  for (i=1; i<=an; i++){
    for(j=1; j<=am; j++){
      *(Ret+offs(an,am,i,j)) = *(A+offs(an,bm,i,j)) - *(B+offs(bn,bm,i,j));
    }
  }
  return Ret;
}

#ifdef STANDALONE
#include <stdlib.h>

int usage(char *progname)
{
    fprintf(stderr,"%s - subtraction matrix B from matrix A\n", progname);
    fprintf(stderr,"Usage:\n%s A [B]\n", progname);
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int argn=1;
    matinfo Ain, Bin;
    scalar ans=0.0;

    if ((argc != 2) && (argc != 3)) {
        usage(argv[0]);
    } 

    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "--help")) usage(argv[0]);
        if (keymatch(argv[argn], "-h")) usage(argv[0]);
        ++argn;
    }
    
    Ain = matinfo_read(argv[1]);
    if (argc == 3) {
	Bin = matinfo_read(argv[2]);
    } else {
	Bin = matinfo_read("-");
    }
    ans = dmatabs(Ain->mat,Ain->n,Ain->m,Bin->mat,Bin->n,Bin->m);

    fprintf(stdout, "### ProgramName: %s %s %s\n", argv[0], 
	    argv[1], (argc==3)?argv[2] : "-");
    fprint_signature(stdout);
    fprintf(stdout,"%g\n", ans);
    return OK;
}
#endif STANDALONE
