/*
 * @(#)matadd.c - matrix operate function 1.02 (Shinichi Asakawa) 21-07-1990
 */
#include <stdlib.h>
#include "matrix_util.h"
#include "fileio_util.h"

/*--- return Ret = A + B ---*/
int matadd(matrix A, int an, int am, 
           matrix B, int bn, int bm, 
           matrix Ret)
{
    int	i,j;
    
    if ((an != bn) || (am != bm)) {
	fprintf(stderr,"Cannot calculate in matadd()\n");
	fprintf(stderr,"A(n*m):n=%d\tm=%d\n", an, am);
	fprintf(stderr,"B(n*m):n=%d\tm=%d\n", bn, bm);
	exit (EXIT_FAILURE);
    }
    
    if (A == NULL) {
	fileio_util_perror("Invalid matrix A in matadd()");
	exit (EXIT_FAILURE);
    }
    
    if (B == NULL) {
	fileio_util_perror("Invalid matrix B in matadd()");
	exit (EXIT_FAILURE);
    }
    
    if (Ret == NULL) {
	fileio_util_perror("Invalid matrix Ret in matadd()");
	exit (EXIT_FAILURE);
    }
    
    for (i=1; i<=an; i++){
	for(j=1; j<=am; j++){
	    *(Ret+offs(an,am,i,j)) += *(A+offs(an,bm,i,j)) + *(B+offs(bn,bm,i,j));
	}
    }
    return OK;
}

/*--- return A += B ---*/
int matADD(matrix A, int an, int am, 
           matrix B, int bn, int bm)
{
    int	i,j;
    
    if ((an != bn) || (am != bm)) {
	fprintf(stderr,"Cannot calculate in matADD()\n");
	fprintf(stderr,"A(n*m):n=%d\tm=%d\n", an, am);
	fprintf(stderr,"B(n*m):n=%d\tm=%d\n", bn, bm);
	exit (EXIT_FAILURE);
    }
    
    if (A == NULL) {
	fileio_util_perror("Invalid matrix A in matADD()");
	exit (EXIT_FAILURE);
    }
    
    if (B == NULL) {
	fileio_util_perror("Invalid matrix B in matADD()");
	exit (EXIT_FAILURE);
    }
    
    for (i=1; i<=an; i++){
	for(j=1; j<=am; j++){
	    *(A+offs(an,bm,i,j)) += *(B+offs(bn,bm,i,j));
	}
    }
    return OK;
}

matrix matAdd(matrix A, int an, int am, 
              matrix B, int bn, int bm)
{
    int	i,j;
    matrix Ret;
    
    if ((an != bn) || (am != bm)) {
	fprintf(stderr,"Cannot calculate in matAdd()\n");
	fprintf(stderr,"A(n*m):n=%d\tm=%d\n", an, am);
	fprintf(stderr,"B(n*m):n=%d\tm=%d\n", bn, bm);
	exit (EXIT_FAILURE);
    }
    
    if (A == NULL) {
	fileio_util_perror("Invalid matrix A in matAdd()");
	exit (EXIT_FAILURE);
    }
    
    if (B == NULL) {
	fileio_util_perror("Invalid matrix B in matAdd()");
	exit (EXIT_FAILURE);
    }
    
    if ((Ret = matinit(an, bm)) == NULL) {
	fileio_util_perror("Could not allocate memory for Ret(matrix) in matAdd()");
	exit (EXIT_FAILURE);
    }
    
    for (i=1; i<=an; i++){
	for(j=1; j<=am; j++){
	    *(Ret+offs(an,am,i,j)) += *(A+offs(an,bm,i,j)) + *(B+offs(bn,bm,i,j));
	}
    }
    return Ret;
}

#ifdef STANDALONE
#include <stdlib.h>

int usage(char *progname)
{
    fprintf(stderr,"%s\n", progname);
    fprintf(stderr,"Usage:\n%s A [B]\n", progname);
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int argn=1;
    matinfo Ain, Bin;

    if ((argc != 2) && (argc != 3)) {
        usage(argv[0]);
    } 

    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "--help")) usage(argv[0]);
        if (keymatch(argv[argn], "-h")) usage(argv[0]);
        ++argn;
    }
    
    Ain = matinfo_read(argv[1]);
    if (argc == 3) {
	Bin = matinfo_read(argv[2]);
    } else {
	Bin = matinfo_read("-");
    }
    matADD(Ain->mat,Ain->n,Ain->m,Bin->mat,Bin->n,Bin->m);

    fprintf(stdout, "### ProgramName: %s %s %s\n", argv[0], 
	    argv[1], (argc==3)?argv[2] : "-");
    fprint_signature(stdout);
    fwritemat(stdout,Ain->mat,Ain->n,Ain->m,"","");
    return OK;
}
#endif STANDALONE
