#include <stdlib.h>
#include <math.h>
#include "matrix_util.h"
#include "fileio_util.h"

/* Ret = a x + b, where a and b is scalar */ 
int matarith(matrix A, int n, int m, double a, double b, matrix Ret)
{
    int i,j;

    if (A == NULL) {
	fileio_util_perror("Could not magnificate in matmag()");
	exit (EXIT_FAILURE);
    }
    if ((n == 0) ||(m == 0)) {
	fileio_util_perror("Invalid matrix size  in matmag()");
	exit (EXIT_FAILURE);
    }

    if (Ret == NULL) {
	fileio_util_perror("Invalid Ret matrix in matMAG()");
	exit (EXIT_FAILURE);
    }
    for (i=1; i<=n; i++) {
        for (j=1; j<=m; j++) {
            *(Ret+offs(n,m,i,j)) = a * *(A+offs(n,m,i,j)) + b;
        }
    }
    return OK;
}

/* X = a X + b, where a and b is scalar */ 
int matARITH(matrix A, int n, int m, double a, double b)
{
    int i,j;

    if (A == NULL) {
	fileio_util_perror("Could not magnificate in matMAG()");
	exit (EXIT_FAILURE);
    }
    if ((n == 0) ||(m == 0)) {
	fileio_util_perror("Invalid matrix size  in matMAG()");
	exit (EXIT_FAILURE);
    }

    for (i=1; i<=n; i++) {
        for (j=1; j<=m; j++) {
            *(A+offs(n,m,i,j)) = a * *(A+offs(n,m,i,j)) + b;
        }
    }
    return OK;
}


/* return Ret (= a x + b), where a and b is scalar */ 
matrix matArith(matrix A, int n, int m, double a, double b) 
{
    int i,j;
    matrix Ret;

    if (A == NULL) {
	fileio_util_perror("Could not magnificate in matMag()");
	exit (EXIT_FAILURE);
    }
    if ((n == 0) ||(m == 0)) {
	fileio_util_perror("Invalid matrix size  in matMag()");
	exit (EXIT_FAILURE);
    }


    if ((Ret = matinit(n,m)) == NULL) {
	fileio_util_perror("Could not allocate memory in matMAG()");
	exit (EXIT_FAILURE);
    }

    for (i=1; i<=n; i++) 
        for (j=1; j<=m; j++) {
            *(Ret+offs(n,m,i,j)) = a * *(A+offs(n,m,i,j)) + b;
        }
    return Ret;
}

#ifdef STANDALONE
#include <stdlib.h>
#include <stdio.h>
#include "fileio_util.h"

int usage (void)
{
    fprintf(stderr,"matarith : arithmetic operation for a matrix\n");
    fprintf(stderr,"Usage matarith {-plus|-minus|-times|-div} <value> [datafile]\n");
    fprintf(stderr,"Options: \n");
    fprintf(stderr," -h : show this help\n");
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    matinfo Ain;
    matrix New;
    double a=1.0, b=0.0;
    int argn = 1;

    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "-help")) {
            usage();
        } else if (keymatch(argv[argn], "-plus")) {
	    b = atof(argv[++argn]);
        } else if (keymatch(argv[argn], "-minus")) {
	    b = -1.0 * atof(argv[++argn]);
        } else if (keymatch(argv[argn], "-times")) {
	    a = atof(argv[++argn]);
        } else if (keymatch(argv[argn], "-div")) {
	    if ( atof(argv[++argn]) == 0.0 ) {
		fprintf(stderr, "Invalid arguments\n");
		usage();
	    }
	    a = 1.0 / atof(argv[argn]);
        } else {
            usage();
        }
        ++argn;
    }
    
    if (argn != argc) { Ain = matinfo_read(argv[argn]); }
    else              { Ain = matinfo_read("-"); }

    if ((Ain->n == 0) || (Ain->m == 0)) usage();
    
    fprintf(stdout, "### ProgramName: matarith -times %g -plus %g %s\n", a, b, (argn != argc)? argv[argn]:"-");
    fprint_signature(stdout);

    New = matArith(Ain->mat,Ain->n,Ain->m,a,b);
    fwritemat(stdout,New,Ain->n,Ain->m,"","");
    return EXIT_SUCCESS;
}
#endif STANDALONE
