#include <strings.h>
#include <stdlib.h>
#include "matrix_util.h"
#include "fileio_util.h"
#include <stdio.h> 
#include <stdlib.h>

char *DEFAULT_OFMT = "%8.5f ";

void usage(void)
{
    fprintf(stderr, "matcat: print matrix with format\n"); 
    fprintf(stderr, "Usage : matcat [-OFMT format] [datafile] ...\n"); 
    fprintf(stderr, "Default OFMT=\"%s\"\n", DEFAULT_OFMT);
    exit (EXIT_FAILURE);
}

char *ofmt = "";

int matcat(char *filename)
{
    matinfo Amat;

    if ( strlen(ofmt) == 0 ) {
        if ((ofmt=(char *)malloc(strlen(DEFAULT_OFMT)+1)) == NULL) {
            fprintf(stderr, "Could not allocate memory in matcat()\n");
            exit (EXIT_FAILURE);
        }
        if ((strcpy(ofmt, DEFAULT_OFMT)) == NULL ) {
            fprintf(stderr, "Could not copy ofmt in matcat()\n");
            exit (EXIT_FAILURE);
        }
        ofmt[strlen(DEFAULT_OFMT)] = '\0'; 
    }
    Amat = matinfo_read(filename);
    fprintf(stdout, "### ProgramName: matcat -OFMT %s %s\n", ofmt, filename);
    fprint_signature(stdout);
    fprintf(stdout, "### Size of matrix: %d %d\n", Amat->n, Amat->m);

    fwritemat(stdout, Amat->mat, Amat->n, Amat->m, "", ofmt);
/*
    free(Amat->mat); free(Amat);
*/
    free(Amat->mat);
    return OK;
}

int main(int argc, char **argv)
{
    int   argn = 1;
    FLAG IsFile = False;

    while (argn < argc) {
        if (keymatch(argv[argn], "--help")) { usage(); } 
        else if (keymatch(argv[argn], "-h")) { usage(); } 
        else if (keymatch(argv[argn], "-OFMT")) {
            argn++;
            if ((ofmt=(char *)malloc(strlen(argv[argn])+2)) == NULL) {
                fprintf(stderr, "Could not allocate memory in %s\n", argv[0]);
                exit (EXIT_FAILURE);
            }
            if (strcpy(ofmt,argv[argn]) == NULL ) {
                fprintf(stderr, "Could not copy ofmt in %s\n", argv[0]);
                exit (EXIT_FAILURE);
            }
            ofmt[strlen(argv[argn])] = ' '; 
            ofmt[strlen(argv[argn])+1] = '\0'; 
        } else {
            IsFile = matcat(argv[argn]);
        }
        argn++;
    }
    if ( ! IsFile )
        return matcat("-");
    else
        return IsFile;
}
