#include <stdlib.h>
#include "matrix_util.h"

int matconvol(matrix ker, int kn, int km, 
              matrix data, int dn, int dm, 
              matrix out)
{
    int i, j, ki, kj, ii, jj;
    int ki_top, kj_left, ki_bottom, kj_right;
    scalar x;

    if ( ker == NULL ) {
        fprintf(stderr, "Invalid kernel matrix in matconvol()\n");
        exit (EXIT_FAILURE);
    }
    if ( data == NULL ) {
        fprintf(stderr, "Invalid data matrix in matconvol()\n");
        exit (EXIT_FAILURE);
    }
    if ( out == NULL ) {
        fprintf(stderr, "Invalid out matrix in matconvol()\n");
        exit (EXIT_FAILURE);
    }

    ki_top    = - (int)floor(kn/2);
    ki_bottom = ki_top + kn;
    kj_left   = - (int)floor(km/2);
    kj_right  = kj_left + km;

    for (i=1; i<=dn; i++) {
        for (j=1; j<=dm; j++) {
            x = 0.0;
            for (ki=ki_top; ki<ki_bottom; ki++) {
                ii = i + ki;
                ii = (ii <= 1) ? 1 : ii;
                ii = (ii >= dn) ? dn : ii;
                for (kj=kj_left; kj<kj_right; kj++) {
                    jj = j + kj;
                    jj = (jj <= 1) ? 1 : jj;
                    jj = (jj >= dm) ? dm : jj;
                    x += *(data+offs(dn,dm, ii             , jj)) * 
                         *(ker +offs(kn,km, ki - ki_top + 1, kj - kj_left + 1));
                }
            }
            *(out+offs(dn,dm,i,j)) = x;
        }
    }
    return OK;
}

int matCONVOL(matrix ker, int kn, int km, 
                 matrix data, int dn, int dm)
{
    matrix Wrk;
    matconvol(ker,kn,km,data,dn,dm,Wrk);
    matcpy(Wrk,dn,dm,data);
    free(Wrk);
    return OK;
}

matrix matConvol(matrix ker, int kn, int km, 
                 matrix data, int dn, int dm)
{
    matrix Ret;
    matconvol(ker,kn,km,data,dn,dm,Ret);
    return Ret;
}


#ifdef STANDALONE
#include <stdlib.h>
#include "fileio_util.h"

int usage(void)
{
    fprintf(stderr,"Usage: matconvol KERNEL [DATA]\n");
    fprintf(stderr," KERNEL : file name of kernel\n");
    fprintf(stderr," DATA : file name of data\n");
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int kn=0,km=0,dn=0,dm=0, argn=1;
    matinfo Kin, Din;
    matrix Kernel, Data, Out;
    
    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "--help")) {
            usage();
	} else if (keymatch(argv[argn], "-h")) {
            usage();
	}
        ++argn;
    }
    
    if ((argc != 3) && (argc != 2)) {
        usage();
    } 
    Kin = matinfo_read(argv[1]);
    if ((Kin->n == 0) || (Kin->m == 0)) {
        fileio_util_perror("matconvol(main) cannnot read kernel");
        exit (EXIT_FAILURE);
    }        
    kn = Kin->n; km = Kin->m; Kernel = Kin->mat;
    
    if (argc == 3) {
        Din = matinfo_read(argv[2]);
    } else {
        Din = matinfo_read("-");
    }
    if ((Din->n == 0) || (Din->m == 0)) {
        fileio_util_perror("matconvol(main) cannnot read data");
        exit (EXIT_FAILURE);
    }        
    dn = Din->n; dm = Din->m; Data = Din->mat;
    Out = matinit(dn,dm);
    matconvol(Kernel, kn, km, Data, dn, dm, Out);

    fprintf(stdout, "### ProgramName: matconvol %s %s\n",
	    argv[1], (argc==3) ? argv[2] : "-" );
    fprint_signature(stdout);

    fwritemat(stdout, Out,dn,dm, "", "%.6g ");
    return OK;
}
#endif STANDALONE
