/*
 * @(#)matgen - return a matrix
 * Author: Shinichi ASAKAWA <asakawa@twcu.ac.jp>
 */
#include <stdlib.h>
#include "matrix_util.h"
#include "fileio_util.h"

matrix matFill(int n, int m, double filling)
{
    matrix Ret;
    int i, j;

    if ( n <= 0 ) {
        fprintf(stderr, "Invalid n(%d) in matFill()\n", n);
        exit (EXIT_FAILURE);
    }
    if ( m <= 0 ) {
        fprintf(stderr, "Invalid m(%d) in matFill()\n", m);
        exit (EXIT_FAILURE);
    }
    if ((Ret = matinit(n,m)) == NULL) {
        fprintf(stderr, "Could not allocate memory in matFill()\n");
        exit (EXIT_FAILURE);
    }
    if ( isnan(filling) != 0 ) {
        fprintf(stderr, "Invalid filing in matfill()\n");
        exit (EXIT_FAILURE);
    }

    for (i=1; i<=n; i++) {
        for (j=1; j<=m; j++) {
            *(Ret+offs(n,m,i,j)) = filling;
        }
    }
    return Ret;
}

int matfill(matrix A, int n, int m, double filling)
{
    int i,j;

    if ( A == NULL ) {
        fprintf(stderr, "Invalid data matrix in matfill()\n");
        exit (EXIT_FAILURE);
    }
    if ( n <= 0 ) {
        fprintf(stderr, "Invalid n(%d) in matfill()\n", n);
        exit (EXIT_FAILURE);
    }
    if ( m <= 0 ) {
        fprintf(stderr, "Invalid m(%d) in matfill()\n", m);
        exit (EXIT_FAILURE);
    }
    /*
      if ( finite(filling) != 0 ) {
      fprintf(stderr, "Invalid filing in matfill()\n");
      exit (EXIT_FAILURE);
      }
    */
    if ( isnan(filling) != 0 ) {
        fprintf(stderr, "Invalid filing in matfill()\n");
        exit (EXIT_FAILURE);
    }
    for (i=1; i<=n; i++) {
        for (j=1; j<=m; j++) {
            *(A+offs(n,m,i,j)) = filling;
        }
    }
    return OK;
}

#ifdef STANDALONE
#include <stdlib.h>

int usage(char *progname)
{
    fprintf(stderr,"%s - return a matrix\n", progname);
    fprintf(stderr,"Usage:\n%s N M filler\n", progname);
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    matrix Out;
    int argn=1;
    int n = 0, m = 0;
    double filler = 0.0;

    if (argc != 4) usage(argv[0]);

    while (argn < argc && argv[argn][0] == '-') {
        if (keymatch(argv[argn], "--help")) { usage(argv[0]); } 
        else if (keymatch(argv[argn], "-h")) { usage(argv[0]); } 
        else { usage(argv[0]); }
        argn++;
    }
    if (sscanf(argv[argn++], "%d", &n) != 1)  usage(argv[0]);
    if (sscanf(argv[argn++], "%d", &m) != 1)  usage(argv[0]);
    if (sscanf(argv[argn++], "%lf", &filler) != 1)  usage(argv[0]);

    if ( n <= 0 ) {
        fprintf(stderr, "Invalid size n(%d) in %s\n",n,argv[0]);
        usage(argv[0]); exit (EXIT_FAILURE);
    }
    if ( m <= 0 ) {
        fprintf(stderr, "Invalid size m(%d) in %s\n",m,argv[0]);
        usage(argv[0]); exit (EXIT_FAILURE);
    }

    Out = matFill(n,m, filler);

    fprintf(stdout, "### ProgramName: %s %d %d %g\n", argv[0], n,m,filler);
    fprint_signature(stdout);
    return fwritemat(stdout, Out, n, m, "", "");
}
#endif STANDALONE
