#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "general.h"
#include "matrix_util.h"
#include "fileio_util.h"

int matHEVISIDE(matrix A, int n, int m, scalar thres)
{
    int i,j;
    double w;

    if ( A == NULL ) {
        fprintf(stderr, "Invalid data matrix in matHEVISIDE()\n");
        exit (EXIT_FAILURE);
    }
    if ( n <= 0 ) {
        fprintf(stderr, "Invalid n(%d)in matHEVISIDE()\n", n);
        exit (EXIT_FAILURE);
    }
    if ( m <= 0 ) {
        fprintf(stderr, "Invalid m(%d) in matHEVISIDE()\n", m);
        exit (EXIT_FAILURE);
    }

    for (i=1;i<=n;i++) {
        for (j=1;j<=m;j++) {
            w = *(A+offs(n,m,i,j));
            *(A+offs(n,m,i,j)) = (w > thres) ? 1.0 : 0.0;
        }
    }
    return OK;
}

matrix matHeviside(matrix A, int n, int m, scalar thres) 
{
    matrix Ret;
    int i,j;
    double w;

    if ( A == NULL ) {
        fprintf(stderr, "Invalid data matrix in matHeviside()\n");
        exit (EXIT_FAILURE);
    }
    if ( n <= 0 ) {
        fprintf(stderr, "Invalid n(%d)in matHeviside()\n", n);
        exit (EXIT_FAILURE);
    }
    if ( m <= 0 ) {
        fprintf(stderr, "Invalid m(%d) in matHeviside()\n", m);
        exit (EXIT_FAILURE);
    }
    if ( (Ret = matinit(n,m)) == NULL ) {
        fprintf(stderr, "Could not allocate memory in matHeviside()\n");
        exit (EXIT_FAILURE);
    }
    
    for (i=1;i<=n;i++){
        for (j=1;j<=m;j++){
            w = *(A+offs(n,m,i,j));
            *(Ret+offs(n,m,i,j)) = (w > thres) ? 1.0 : 0.0;
        }
    }
    return Ret;
}

int matheviside(matrix A, int n, int m, scalar thres, matrix Ret) 
{
    int i,j;
    double w;

    if ( A == NULL ) {
        fprintf(stderr, "Invalid data matrix in matheviside()\n");
        exit (EXIT_FAILURE);
    }
    if ( n <= 0 ) {
        fprintf(stderr, "Invalid n(%d)in matheviside()\n", n);
        exit (EXIT_FAILURE);
    }
    if ( m <= 0 ) {
        fprintf(stderr, "Invalid m(%d) in matheviside()\n", m);
        exit (EXIT_FAILURE);
    }
    
    for (i=1;i<=n;i++){
        for (j=1;j<=m;j++){
            w = *(A+offs(n,m,i,j));
            *(Ret+offs(n,m,i,j)) = (w > thres) ? 1.0 : 0.0;
        }
    }
    return OK;
}

#ifdef STANDALONE
#include <stdlib.h>
#include <time.h>
long loadingtime;

int usage(char *prgname)
{
    fprintf(stderr,"%s\n", prgname);
    fprintf(stderr,"Usage:\n%s [-thres <double>] [Input]\n", prgname);
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int argn=1;
    char *filename;
    matinfo Input;
    matrix Out;
    scalar thres = 0.0;

    if ((argc < 2) && (4 < argc)) { usage(argv[0]); } 

    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "--help")) usage(argv[0]);
        if (keymatch(argv[argn], "-h")) usage(argv[0]);
        if (keymatch(argv[argn], "-thres")) {
#ifdef SCALAR_AS_FLOAT			
        if (sscanf(argv[++argn], "%g", &thres) != 1)  usage(argv[0]);
#else
        if (sscanf(argv[++argn], "%lg", &thres) != 1)  usage(argv[0]);
#endif
    }
        ++argn;
    }

    if (argn != argc) { 
        if ((filename = (char *)malloc(strlen(argv[argn])+1)) == NULL) {
            fprintf(stderr, "Could not allocate memory in main() of matheviside\n"); 
            exit (EXIT_FAILURE);
        }
        if (strcpy(filename,argv[argn]) == NULL) {
            fprintf(stderr, "Could not strcpy in main() of matheviside()\n");
            exit (EXIT_FAILURE);
        }
        filename[strlen(argv[argn])] = '\0';
        argn++;
    } else {
        filename = (char *)malloc(2);
        filename[0]='-'; filename[1]='\0';
    }
    if (argn != argc) usage(argv[0]);


    Input = matinfo_read(filename);
    Out = matHeviside(Input->mat, Input->n, Input->m, thres);

    fprintf(stdout, "### ProgramName: %s -thres %g %s\n", argv[0], thres, filename);
    fprint_signature(stdout);
    fwritemat(stdout, Out, Input->n, Input->m, "", "");
    return OK;
}
#endif STANDALONE
