/*
 * @(#)matprd.c - matrix operate function 1.02 (Shinichi Asakawa) 21-07-1990
 */
#include <stdlib.h>
#include "matrix_util.h"
#include "fileio_util.h"

/*--- ret = A * B ---*/
int matprd(matrix A, int an, int am, 
           matrix B, int bn, int bm, matrix Ret) 
{
    int i,j,k;
    double w = 0.0;
    
    if (am != bn) {
        fprintf(stderr,"Cannot calculate in matprd()\n");
        fprintf(stderr,"A(n*m):n=%d\tm=%d\n",an,am);
        fprintf(stderr,"B(n*m):n=%d\tm=%d\n",bn,bm);
        exit(0);
    }

    if ( A == NULL ) {
        fileio_util_perror("Invalid matrix A in matprd()");
        exit (EXIT_FAILURE);
    }

    if ( B == NULL ) {
        fileio_util_perror("Invalid matrix A in matprd()");
        exit (EXIT_FAILURE);
    }

    if ( Ret == NULL ) {
        fileio_util_perror("Invalid matrix Ret in matprd()");
        exit (EXIT_FAILURE);
    }

    for (i=1; i<=an; i++) {
        for (j=1; j<=bm; j++) {
            w = 0.0;
            for (k=1; k<=am; k++) {
                w += *(A+offs(an,am,i,k)) * *(B+offs(bn,bm,k,j));
            }
            *(Ret+offs(an,bm,i,j)) = w;
        }
    }
    return OK;
}

/*--- return A *= B ---*/
int matPRD(matrix A, int an, int am, 
           matrix B, int bn, int bm)
{
    int i,j,k;
    matrix Wrk;
    double w=0.0;

    if (am != bn) {
        fprintf(stderr,"Cannot calculate in matPRD()\n");
        fprintf(stderr,"A(n*m):n=%d\tm=%d\n", an, am);
        fprintf(stderr,"B(n*m):n=%d\tm=%d\n", bn, bm);
        exit (EXIT_FAILURE);
    }

    if ( A == NULL ) {
        fileio_util_perror("Invalid matrix A in matPRD()");
        exit (EXIT_FAILURE);
    }

    if ( B == NULL ) {
        fileio_util_perror("Invalid matrix A in matPRD()");
        exit (EXIT_FAILURE);
    }

    if ( (Wrk = matinit(an,bm)) == NULL ) {
        fileio_util_perror("Could not allocate memory in matPRD()");
        exit (EXIT_FAILURE);
    }
    for (i=1; i<=an; i++) {
        for (j=1; j<=bm; j++) {
            w = 0.0;
            for (k=1; k<=am; k++) {
                w += *(A+offs(an,am,i,k)) * *(B+offs(bn,bm,k,j));
            }
            *(Wrk+offs(an,bm,i,j)) = w;
        }
    }
    for (i=1; i<=an; i++) {
        for (j=1; j<=bm; j++) {
            *(A+offs(an,am,i,j)) = *(Wrk+offs(an,am,i,j));
        }
    }
    free(Wrk);
    return OK;
}

/* return Ret (= A * B) */
matrix matPrd(matrix A, int an, int am, 
              matrix B, int bn, int bm)
{
    int i,j,k;
    matrix Ret;
    double w;

    if (am != bn) {
        fprintf(stderr,"Cannot calculate in matPrd()\n");
        fprintf(stderr,"A(n*m):n=%d\tm=%d\n", an, am);
        fprintf(stderr,"B(n*m):n=%d\tm=%d\n", bn, bm);
        exit (EXIT_FAILURE);
    }

    if ( A == NULL ) {
        fileio_util_perror("Invalid matrix A in matPrd()");
        exit (EXIT_FAILURE);
    }

    if ( B == NULL ) {
        fileio_util_perror("Invalid matrix A in matPrd()");
        exit (EXIT_FAILURE);
    }

    if ((Ret = matinit(an, bm)) == NULL) {
        fileio_util_perror("Could not allocate memory for Ret(matrix) in matAdd()");
        exit (EXIT_FAILURE);
    }

    for (i=1; i<=an; i++) {
        for (j=1; j<=bm; j++) {
            w = 0.0;
            for (k=1; k<=am; k++) {
                w += *(A+offs(an,am,i,k)) * *(B+offs(bn,bm,k,j));
            }
            *(Ret+offs(an,bm,i,j)) = w;
        }
    }
    return Ret;
}

#ifdef STANDALONE
#include <stdlib.h>

int usage(char *prgname)
{
    fprintf(stderr,"%s\n", prgname);
    fprintf(stderr,"Usage:\n%s A [B]\n", prgname);
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int argn=1;
    matinfo Ain, Bin;
    matrix Out;

    if ((argc != 2) && (argc != 3)) {
        usage(argv[0]);
    } 

    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "--help")) usage(argv[0]);
        if (keymatch(argv[argn], "-h")) usage(argv[0]);
        ++argn;
    }
    
    Ain = matinfo_read(argv[1]);
    if (argc == 3) {
        Bin = matinfo_read(argv[2]);
    } else {
        Bin = matinfo_read("-");
    }
    Out = matPrd(Ain->mat,Ain->n,Ain->m,Bin->mat,Bin->n,Bin->m);

    fprintf(stdout, "### ProgramName: %s %s %s\n", argv[0], 
            argv[1], (argc==3)?argv[2] : "-");
    fprint_signature(stdout);

    fwritemat(stdout, Out, Ain->n, Bin->m, "", "");
    return OK;
}
#endif STANDALONE
