/*
 * @(#)matresize.c - matrix operate function 1.02 (Shinichi Asakawa) 21-07-1990
 */
#include <stdlib.h>
#include "matrix_util.h"
#include "fileio_util.h"

int matresize(matrix O, int n, int m, 
	      matrix Ret, int rn, int rm)
{
    int	t,i,j;
    int total = rn * rm;

    if ((n == 0) || (m == 0) || (O == NULL) ) {
	fprintf(stderr,"Invalid input matrix in matresize()\n");
	exit (EXIT_FAILURE);
    }
    if ((rn == 0) || (rm == 0) || (Ret==NULL)) {
	fprintf(stderr,"Invalid output matrix in matresize()\n");
	exit (EXIT_FAILURE);
    }

    if ( (rn * rm) < (n * m) ) {
	fprintf(stderr,"#### Original matrix is larger than target matrix at matresize()\n");
    }

    for (i=1,j=1,t=1; t<= total; j++,t++) {
        if ( t > ( n * m ) ) { *(Ret+offs(rn,rm,i,j)) = 0.0;
        } else {               *(Ret+offs(rn,rm,i,j)) = *(O+t-1); }
        if ((t % rm) == 0) { 
            j=0; i++;
        }
    }
    return OK;
}


matrix matResize(matrix A, int an, int am, 
		 int rn, int rm)
{
    matrix Ret;
    
    if ((an == 0) || (am == 0) || (A == NULL)) {
	fprintf(stderr, "Invalid input matrix in matResize()\n");
	exit (EXIT_FAILURE);
    }
    if ((rn == 0) || (rm == 0)) {
	fprintf(stderr, "Invalid output matrix in matResize()\n");
	exit (EXIT_FAILURE);
    }
    if ((Ret = matinit(rn, rm)) == NULL) {
	fileio_util_perror("Could not allocate memory for Ret in matResize()");
	exit (EXIT_FAILURE);
    }
    matresize(A,an,am, Ret,rn,rm);
    return Ret;
}

#ifdef STANDALONE
#include <stdlib.h>

int usage(char *prgname)
{
    fprintf(stderr,"%s - Resize matrix\n", prgname);
    fprintf(stderr,"Usage:\n%s N M [datafile]\n", prgname);
    fprintf(stderr, "     Arguments: N rows\n");
    fprintf(stderr, "                M cols\n");
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int argn=1;
    char *filename;
    int n = 0, m = 0;
    matinfo Ain;
    matrix B;

    if ((argc != 3) && (argc != 4)) { usage(argv[0]); } 

    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "--help")) usage(argv[0]);
        if (keymatch(argv[argn], "-h")) usage(argv[0]);
        ++argn;
    }

    n = atoi(argv[1]); m = atoi(argv[2]);

    if ((n == 0) || (m == 0)) {
	fprintf(stderr, "Invaliad parameter(s) in matresize main()\n");
	exit (EXIT_FAILURE);
    }
    if (argc == 4) {
	if ((filename = (char *)malloc(strlen(argv[3])+1)) == NULL) {
	    fprintf(stderr, "Could not allocate memory in matresize()\n");
	    exit (EXIT_FAILURE);
	}
	if (strcpy(filename,argv[3]) == NULL) {
	    fprintf(stderr, "Could not strcpy in matresize()\n");
	    exit (EXIT_FAILURE);
	}
	filename[strlen(argv[3])] = '\0';
    } else {
        if ((filename = (char *)malloc(2)) == NULL) {
            fprintf(stderr, "Could not allocate memory in matresize()\n");
            exit (EXIT_FAILURE);
        }
        filename[0]='-'; filename[1]='\0';
    }
    Ain = matinfo_read(filename);

    B = matinit(n,m);
    matresize(Ain->mat, Ain->n, Ain->m, B, n, m);

    fprintf(stdout, "### ProgramName: %s %d %d %s\n", 
	    argv[0], n, m, filename);
    fprint_signature(stdout);
    fwritemat(stdout,B, n, m, "", "");
    return OK;
}
#endif STANDALONE
