/*
 *  @(#) matstandard - Standarize each column 
 */
#include <stdlib.h>
#include <math.h>
#include "general.h"
#include "matrix_util.h"

int matstandard(matrix A, int n, int m, matrix Ret)
{
	if ( A == NULL ){
		fprintf(stderr,"Could not allocate memory in matstandard()\n");
		exit (EXIT_FAILURE);
	}
	if ( Ret == NULL ){
		fprintf(stderr,"Could not allocate memory in matstandard()\n");
		exit (EXIT_FAILURE);
	}
	Ret = matStandard(A,n,m);
	return OK;
}

int matSTANDARD(matrix A, int n, int m)
{
	matrix Tmp;
	
	if ( A == NULL ){
		fprintf(stderr,"Could not allocate memory in matSTANDARD()\n");
		exit (EXIT_FAILURE);
	}
	Tmp = matStandard(A,n,m);
	matcpy(Tmp,n,m,A);
	free(Tmp);
	return OK;
}

matrix matStandard(matrix A, int n, int m)
{
	int i,j;
	double v;
	matrix Ret = matinit(n,m);
	matrix mean = bstat_mean(A, n, m);
	matrix var = bstat_sd(A,n,m);
	
	if ( A == NULL ){
		fprintf(stderr, "Could not allocate memory in matStandard()\n");
		exit (EXIT_FAILURE);
	}
	if ( Ret == NULL ){
		fprintf(stderr, "Could not allocate memory in matStandard()\n");
		exit (EXIT_FAILURE);
	}
	
	for (j=1; j<=m; j++) {
		if ( *(var+offs(1,m,1,j)) < eps ) v = 1.0;
		else	v = *(var+offs(1,m,1,j));
		
		for (i=1; i<=n; i++) {
			*(Ret+offs(n,m,i,j)) 
				= *(A+offs(n,m,i,j)) - *(mean+offs(1,m,1,j));
			*(Ret+offs(n,m,i,j)) /= v;
		}
	}
	return Ret;
}

#ifdef STANDALONE
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "fileio_util.h"

int usage(char *prgname)
{
	printf("%s - Standard each column in the input matrix\n", prgname);
	printf("Usage %s [options] [datafile]\n", prgname);
	printf("options: \n");
	printf(" --help   : show this help\n");
	exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
	int  argn = 1;
	matinfo Ain;
	matrix Ret;
	char *filename;
	
	while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
		if (keymatch(argv[argn], "--help")) {
			usage(argv[0]);
		} else if (keymatch(argv[argn], "-h")) {
			usage(argv[0]);
		} else {
			usage(argv[0]);
		}
		++argn;
	}
	
	if (argn != argc) { 
		if ((filename = (char *)malloc(strlen(argv[argn])+1)) == NULL) {
			fprintf(stderr, "Could not allocate memory in main() of matstandard\n"); 
			exit (EXIT_FAILURE);
		}
		if (strcpy(filename,argv[argn]) == NULL) {
			fprintf(stderr, "Could not strcpy in matstandard()\n");
			exit (EXIT_FAILURE);
		}
		filename[strlen(argv[argn])] = '\0';
		argn++;
	} else {
		filename = (char *)malloc(2);
		filename[0]='-'; filename[1]='\0';
	}
	if (argn != argc) usage(argv[0]);
	Ain = matinfo_read(filename);
	if ( (Ain->n == 0) || (Ain->m == 0) ) usage(argv[0]);
	
	fprintf(stdout, "### ProgramName: %s %s\n", argv[0], filename);
	fprint_signature(stdout);
	
	Ret = matStandard(Ain->mat, Ain->n, Ain->m);
	fwritemat(stdout, Ret, Ain->n, Ain->m, "", "");
	return OK;
}
#endif STANDALONE
