/*
 * @(#)matt.c - matrix operate function 1.02 21-07-1990
 * by Shinichi ASAKAWA <asakawa@twcu.ac.jp>
 */
#include <stdlib.h>
#include "matrix_util.h"
#include "fileio_util.h"

int matt(matrix A, int m, int n, matrix Ret)
{
    int i,j;

    if ( A == NULL ) {
	fileio_util_perror("Invalid matrix in matt()");
	exit (EXIT_FAILURE);
    }
    if ( (m == 0) || (n == 0) ) {
	fileio_util_perror("Invalid matrix size in matt()");
	exit (EXIT_FAILURE);
    }

    if ( Ret == NULL ) {
	fileio_util_perror("Invalid Ret matrix in matt()");
	exit (EXIT_FAILURE);
    }	    

    for (i=1; i <= m; i++) {
	for (j=1; j <= n; j++) {
	    *(Ret+offs(n,m,j,i)) = *(A+offs(m,n,i,j));
	}
    }
    return OK;
}

matrix matT(matrix A, int m, int n)
{
    matrix rA;
    int	i,j;

    if ( A == NULL ) {
	fileio_util_perror("Invalid matrix in matT()");
	exit (EXIT_FAILURE);
    }
    if ( (m == 0) || (n == 0) ) {
	fileio_util_perror("Invalid matrix size in matT()");
	exit (EXIT_FAILURE);
    }

    if ((rA = matinit(n,m)) == NULL) {
	fileio_util_perror("Could not allocate memory in matT()");
	exit (EXIT_FAILURE);
    }
    for (i=1; i <= m; i++) {
	for (j=1; j <= n; j++) {
	    *(rA+offs(n,m,j,i)) = *(A+offs(m,n,i,j));
	}
    }
    return rA;
}

#ifdef STANDALONE
#include <stdio.h>

int usage(void)
{
    fprintf(stderr,"matt\n");
    fprintf(stderr,"Usage:\n matt [options] [file]\n");
    fprintf(stderr,"options: \n");
    fprintf(stderr," --help : show this help\n");
    exit (EXIT_FAILURE);
}	

int main(int argc, char **argv)
{
    int argn = 1;
    matrix Out;
    matinfo in;

    if (argc > 2) usage();
    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
	if (keymatch(argv[argn], "--help")) { usage(); } 
	else if (keymatch(argv[argn], "-h")) { usage(); } 
	else { usage(); }
	++argn;
    }

    if (argc == 2) { in = matinfo_read(argv[1]); }
    else           { in = matinfo_read("-"); } 

    if ( (in->n == 0) || (in->m == 0) || (in->mat == NULL) ) {
        fileio_util_perror("Could not read matrix in matt()\n");
        usage();
    }
    Out = matT(in->mat, in->n, in->m);
    fprintf(stdout, "### ProgramName: %s %s (transposed matrix)\n",argv[0],(argc==2) ? argv[1] : "-");
    fprint_signature(stdout);
    fwritemat(stdout, Out, in->m, in->n, "", "");
    return OK;
}
#endif STANDALONE
