#include <stdlib.h>
#include "general.h"
#include "matrix_util.h"
#include "fileio_util.h"

int intswap(int *a, int *b)
{
    int w;
    w = *a; *a = *b; *b = w;
    return OK;
}

int mattrim(matrix O,   /* An origninal matrix to be trimmed */
            int n,      /* rows of original matrix */
            int m,      /* columns of original matrix */ 
            int top,    /* upper bound of trimmed matrix */ 
            int left,   /* left bound of trimmed matrix */ 
            int bottom, /* lower boud */ 
            int right,  /* right boud of */
            matrix Ret  /* matrix to be returned */)
{
    int i, j, dsn, dsm;

    top = (top < 1 ) ? 1 : top;
    left = (left < 1 ) ? 1 : left;
    bottom = (bottom < 1 ) ? 1 : bottom;
    right = (right < 1 ) ? 1 : right;
    if ( bottom < top ) { intswap(&top, &bottom); }
    if ( right < left ) { intswap(&left, &right); }

    if ( top > n ) {
	fprintf(stderr, "top value(%d) exceed the original size(%d) in mattrim()\n",
		top, n);
	exit (EXIT_FAILURE);
    }
    if ( bottom > n ) {
	fprintf(stderr, "bottom value(%d) exceed the original size(%d) in mattrim()\n",
		bottom, n);
	exit (EXIT_FAILURE);
    }
    if ( left > m ) {
	fprintf(stderr, "left value(%d) exceed the original size(%d) in mattrim()\n",
		left, m);
	exit (EXIT_FAILURE);
    }
    if ( right > m ) {
	fprintf(stderr,"right value(%d) exceed the original size(%d) in mattrim()\n",
		right,m);
	exit (EXIT_FAILURE);
    }
    
    dsn = (bottom - top)+1;
    dsm = (right - left)+1;
    if (Ret == NULL) {
	fileio_util_perror("Could not allocate memory in mattrim()");
	exit (EXIT_FAILURE);
    }

    for (i=1; i<=n; i++) {
	if ((top <= i) && (i <= bottom)) {
	    for (j=1; j<=m; j++) {
		if ((left <= j) && (j <= right)) {
		    *(Ret+offs(dsn,dsm,i-top+1,j-left+1)) = *(O+offs(n,m,i,j));
		}
	    }
	}
    }
    return OK;
}

matrix matTrim(matrix O, /* An origninal matrix to be trimmed */
               int n,    /* rows of original matrix */
               int m,    /* columns of original matrix */ 
               int top,   /* upper bound of trimmed matrix */ 
               int left,   /* left bound of trimmed matrix */ 
               int bottom,   /* lower boud */ 
               int right    /* right boud of */ )
{
    int dsn, dsm;
    matrix Ret;
    
    top = (top < 1 ) ? 1 : top;
    left = (left < 1 ) ? 1 : left;
    bottom = (bottom < 1 ) ? 1 : bottom;
    right = (right < 1 ) ? 1 : right;
    if ( bottom < top ) { intswap(&top, &bottom); }
    if ( right < left ) { intswap(&left, &right); }

    dsn = (bottom - top)+1;
    dsm = (right - left)+1;
    if ((Ret = matinit(dsn, dsm))==NULL) {
	fileio_util_perror("Could not allocate memory in matTrim()");
	exit (EXIT_FAILURE);
    }
    mattrim(O,n,m,top,left,bottom,right,Ret);
    return Ret;
}

#ifdef STANDALONE
#include <stdlib.h>

int usage(char *progname)
{
    fprintf(stderr,"%s\n", progname);
    fprintf(stderr,"Usage %s sn sm en em [datafile]\n", progname);
    fprintf(stderr," sn : cell number of upper (int)\n");
    fprintf(stderr," sm : cell number of left (int) \n");
    fprintf(stderr," en : cell number of lower (int)\n");
    fprintf(stderr," em : cell number of right (int)\n");
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    matrix Out;
    matinfo DataInfo;
    int sn=0, sm=0, en=0, em=0;
    int argn=1;

    if ((argc != 5) && (argc != 6)) {
        usage(argv[0]);
    } 

    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "--help")) usage(argv[0]);
        if (keymatch(argv[argn], "-h")) usage(argv[0]);
        ++argn;
    }
    
    if (sscanf(argv[1], "%d", &sn) != 1)  usage(argv[0]);
    if (sscanf(argv[2], "%d", &sm) != 1)  usage(argv[0]);
    if (sscanf(argv[3], "%d", &en) != 1)  usage(argv[0]);
    if (sscanf(argv[4], "%d", &em) != 1)  usage(argv[0]);

    argn = 5;
    if (argn != argc) { 
        DataInfo = matinfo_read(argv[argn]);
    } else { 
        DataInfo = matinfo_read("-");
    }

    if ((DataInfo->n == 0) || (DataInfo->m == 0))
	usage(argv[0]);
    if ( sn > en ) {
	fprintf(stderr, "Invalid argument sn=%d, en=%d\n", sn, en);
	exit (EXIT_FAILURE);
    }
    if ( sm > em ) {
	fprintf(stderr, "Invalid argument sm=%d, em=%d\n", sm, em);
	exit (EXIT_FAILURE);
    }

    Out = matTrim(DataInfo->mat,DataInfo->n,DataInfo->m,sn,sm,en,em);

    fprintf(stdout, "### ProgramName: %s %d %d %d %d %d %d %s\n",
	    argv[0], DataInfo->n, DataInfo->m,sn,sm,en,em, 
	    (argn!=argc) ? argv[argn] : "-");
    fprint_signature(stdout);
    fwritemat(stdout, Out, en-sn+1, em-sm+1, "", "%g ");
    return OK;
}
#endif STANDALONE
