/*  Calculate eigen values by power method */
#include <stdlib.h>
#include <math.h>
#include "general.h"
#include "matrix_util.h"
#include <stdlib.h>
#include "fileio_util.h"

int usage(char *prgname)
{
    printf("%s  - Simple Eigenvalue\n", prgname);
    printf("Usage %s [options] [datafile]\n", prgname);
    printf("options: \n");
    printf(" -p      : number of eigenvalues to be extracted\n");
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int m=0, n=0, p=0, argn = 1;
    matinfo Ain;
    matrix A, eig, evec, printEig, printEvec;
    
    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "-help")) {
            usage(argv[0]);
        } else if (keymatch(argv[argn], "-p")) {
            ++argn;
            if (argn == argc || sscanf(argv[argn], "%d", &p) != 1)
                usage(argv[0]);
        } else {
            usage(argv[0]);
        }
        ++argn;
    }
    
    if (argn != argc) {
        Ain = matinfo_read(argv[argn]);
        ++argn;
    } else {
        Ain = matinfo_read("-");
    }
    if (argn != argc)
        usage(argv[0]);
    
    A = Ain->mat;
    n = Ain->n;
    m = Ain->m;
    
    if ((n == 0) || (m == 0)) usage(argv[0]);
    if (n != m) usage(argv[0]);

    if (p == 0) p = m;
    
    eig = matinit(m,1);
    evec = matinit(m,m);
    fprintf(stdout, "### Eigen value problem of a matrix\n");
    fprint_signature(stdout);
    fprintf(stdout, "### Rows: %d\n",n);
    fprintf(stdout, "### Columns: %d\n",m);
    fprintf(stdout, "### EigenvaluesToBeExtracted: %d\n",p);
    
    eigen_pm(A,eig,evec,m,p);
    
    if ( p == m ){
            fwritemat(stdout, eig,  m,1, "### EigenValues:\n","%.6g ");
            fwritemat(stdout, evec, m,m, "### EigenVectors:\n", "%.6g ");
    } else {
            printEig = matTrim(eig, m,1,1,1,p,1);
            printEvec = matTrim(evec, m,m, 1,1, m,p);
            fwritemat(stdout, printEig,  p,1, "### EigenValues:\n","%.6g ");
            fwritemat(stdout, printEvec, m,p, "### EigenVectors:\n", "%.6g ");
    }
    return OK;
}
