#include <math.h>
#include "general.h"
#include "rand_gen.h"

/* extern double normal_rand(); */

double gamma_rand1 (int two_a)  /* $B%,%s%^J,I[(B */
{
    int i;
    double x, r;

    x = 1;
    for (i = two_a / 2; i > 0; i--) x *= MTgenrand_f();
    x = -log(x);
    if ((two_a & 1) != 0) {  /* if two_a is odd */
        r = normal_rand();  
	x += 0.5 * r * r;
    }
    return x;
}

double gamma_rand2 (double a)  /* $B%,%s%^J,I[(B, a > 0 */
{
    double t, u, x, y;

    if ( a > 1.0 ) {
        t = sqrt(2.0 * a - 1.0);
        do {
            do {
                /* $B<!$N#49T$O(B y = tan(PI * MTgenrand_f()) $B$HF1CM(B */
                do {
                    x = 1.0 - MTgenrand_f();  
		    y = 2.0 * MTgenrand_f() - 1.0;
                } while (x * x + y * y > 1.0);
                y /= x;
                x = t * y + a - 1.0;
            } while (x <= 0);
            u = (a - 1.0) * log(x / (a - 1.0)) - t * y;
        } while (u < -50.0 || MTgenrand_f() > (1 + y * y) * exp(u));
    } else {
        t = E / (a + E);
        do {
            if (MTgenrand_f() < t) {
                x = pow(MTgenrand_f(), 1 / a);  y = exp(-x);
            } else {
                x = 1 - log(MTgenrand_f());  y = pow(x, a - 1);
            }
        } while (MTgenrand_f() >= y);
    }
    return x;
}

double gamma_rand(double p) {
    return gamma_rand2(p);
}

/**********************************************************/
#ifdef STANDALONE
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

long loadingtime;
unsigned int SEED = 0;

void Usage (char *prg) {
  fprintf(stderr,"Usage:\n");
  fprintf(stderr,"%s POWER N [SEED]\n", prg);
  fprintf(stderr,"options  POWER: parameter for gamma distribution\n");
  fprintf(stderr,"         N: number of data to be generated(int)\n");
  fprintf(stderr,"         SEED: seed of random number(long int)\n");
  exit (EXIT_FAILURE);
}

int main (int argc, char **argv) {
    int i, max = 0;
    double p = 0.0;

    SEED = (unsigned)time(&loadingtime);

    switch (argc) {
    case 3:
        p = atof(argv[1]);
        max = (unsigned)atoi(argv[2]);
        break;
    case 4:	
        p = atof(argv[1]);
        max = (unsigned)atoi(argv[2]);
        SEED = atof(argv[3]); 
        break;
    default:
	Usage(argv[0]); exit (EXIT_FAILURE);
    }
    if (max == 0)     Usage(argv[0]);
    if (p == 0.0)     Usage(argv[0]);
    MTsgenrand(SEED);

    fprintf(stdout, "### ProgramName: %s %g %d %d\n", argv[0], p, max, SEED);
    fprintf(stdout, "### Creator: Shinichi ASAKAWA <asakawa@twcu.ac.jp>\n");
    (void)time(&loadingtime);
    fprintf(stdout, "### CreationDate: %s", ctime(&loadingtime));

    for (i = 0; i < max; i++) {
	printf("%10.5f\n", gamma_rand(p));
    }
    return EXIT_SUCCESS;
}
#endif STANDALONE
