#include <math.h>
#include "general.h"
#include "rand_gen.h"

int poisson_rand(double lambda)  /* Poisson ($B%]%"%=%s(B) $BJ,I[(B */
{
    int k;

    lambda = exp(lambda) * MTgenrand_f();
    k = 0;
    while (lambda > 1) {
	lambda *= MTgenrand_f();  k++;
    }
    return k;
}

/**********************************************************/
#ifdef STANDALONE
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

long loadingtime;
unsigned int SEED = 0;

void Usage (char *prg) {
    fprintf(stderr,"Usage:\n");
    fprintf(stderr,"%s lambda MAX [SEED]\n", prg);
    fprintf(stderr,"arguments  lambda: parameter for Poisson distribution(double)\n");
    fprintf(stderr,"           MAX: number of data to be generated(int)\n");
    fprintf(stderr,"           SEED: seed of random number(long int)\n");
    exit (EXIT_FAILURE);
}

int main (int argc, char **argv) {
    int i, max = 0;
    double lambda=0.0;

    SEED = (unsigned)time(&loadingtime);

    switch (argc) {
    case 3:
        lambda = atof(argv[1]);
        max = (unsigned)atoi(argv[2]);
        break;
    case 4:
        lambda = atof(argv[1]);
        max = (unsigned)atoi(argv[2]);
        SEED = atof(argv[3]); 
        break;
    default:
	Usage(argv[0]); exit (EXIT_FAILURE);
    }
    if (max == 0) Usage(argv[0]);
    if (lambda == 0.0)  Usage(argv[0]);
    MTsgenrand(SEED);

    fprintf(stdout, "### ProgramName: %s %g %d %d\n", argv[0], lambda, max, SEED);
    fprintf(stdout, "### Creator: Shinichi ASAKAWA <asakawa@twcu.ac.jp>\n");
    (void)time(&loadingtime);
    fprintf(stdout, "### CreationDate: %s", ctime(&loadingtime));

    for (i = 0; i < max; i++) {
	printf("%7d\n", poisson_rand(lambda));
    }
    return EXIT_SUCCESS;
}
#endif STANDALONE
