/*	Calculate the normalized z score from the probability
 *		Written by S.Asakawa 1993/05/04 */
#include <math.h>
#include "general.h"

/* Gulliksen-Tucker ζΨ p ɸಽ z 
   ưʳؤΤBASIC硤ɵסǲ1983, p.59
   p = 0.010.99 ϰϤ 3 ʾ٤ */
double p2z(double p)
{
	double u, a1, a2;
	u = p - 0.5;
	a1 = u * (( 11.2502 * u * u - 12.2043 ) * u * u + 2.5101);
	a2 = ( 7.9587 * u * u - 5.8742 ) * u * u + 1.0;
	return a1/a2;
}

#ifdef TEST
#include	<stdio.h>
#include	<stdlib.h>

void usage(char *prgname)
{
    fprintf(stderr,"%s - Convert probablity to the corresponding z score of the standard normal distribution\n", prgname);
    fprintf(stderr,"usage: %s probability\n", prgname);
    exit (EXIT_FAILURE);
}

int main( int argc, char **argv )
{
	double p;

	if( argc != 2 )	usage(argv[0]);
	sscanf(argv[1],"%lg",&p);

	if ( p < 0 || p > 1.0 ) {
		fprintf(stderr, "Invalid probability(%g) must be between 0 and 1\n", p);
		usage(argv[0]);
  }
	fprintf(stdout,"%g\n", p2z(p));
	return EXIT_SUCCESS;
}
#endif
