#include <math.h>
#include <pazu.h>

double p2z(double p)
/* Gulliksen-TuckerζΨ𤫤ɸಽ */
/* ưʳؤΤBASIC硤ɵסǲ񡤣,p59  */
/* =.01.99 ϰϤǣʾ٤                       */
{
	double  u,a1,a2;
	u = p - 0.5;
	a1 = u * (( 11.2502 * u * u - 12.2043 ) * u * u + 2.5101);
	a2 = ( 7.9587 * u * u - 5.8742 ) * u * u + 1.0;
	return (a1/a2);
}

double z2p(double y)
/* Hastings(1955)ζȤäƣΨ  */
/* Żҷ׻Τζ׻޽ѡˣ    */
/* ưʳؤΤBASIC硤ɵסǲ񡤣,p59 */
/* ٤Ͼʲ                                  */
{
	double  x1=0.196854, x2=0.115195, x3=0.000344, x4=0.019527,a,b,c,d,z;

	z = fabs(y);
	a = (x4 * z + x3) * z + x2;
	b = a * z + x1;
	c = b * z + 1;
	d = 0.5 * pow(c, -4.0);
	return ( (y >= 0) ? (1 - d) : d );
}

double z2w(double z)
/* MeullerUbanٽţ */
{
	double  y, p, ret;
	y = exp( -0.5 * z * z ) / sqrt( 2.0 * Pi);
	p = z2p(z);
	ret = y * y / ( p * ( 1.0 - p ));
	return ret;
}

double p2w(double p)
/* Ψ p MeullerUbanٽţ */
{
	double  z, ret;
	z = p2z(p);
	ret = z2w(z);
	return ret;
}

double prob_normal( double x )
/* N(0,1) ˽Ψѿ0 < z < x ϰϤȤΨ */
{
	int i;
	double lastp, p, temp;

	x = x * x;
	p = sqrt( x / ( 2.0 * Pi * exp(x)));
	temp = p;
	i = 3;
	if( temp > 0.0 ){
		do{
			lastp = p;
			temp *= x / (double)i;
			p += temp;
			i += 2;
		}
		while(fabs(p - lastp) > Eps);
	}
	return p;
}

double prob_binominal(int n, int k, double p)
/* ʬۣ¡ʣˤ˽ΨѿϰϤȤΨ */
/* ʤΨݤΩˣԤʤäƣ */
/* Ψ */
{
	register int i;
	double q, ret, temp;

	ret = temp = exp( (double)n * log( q = 1.0 - p));
	for(i=1;i<=k;i++){
		temp *= ((double)(n - i + 1)) * p / ((double)i * q);
		ret += temp;
	}
	return ret;
}

double prob_chisquare(int df, double l)
/* ʬۤ˽ΨѿϰϤȤΨ */
{
	double	p,lastp,loggamma,temp;

	loggamma = (df % 2 == 1) ? log(Pi)/2 : 0;
	temp = (double)df / 2.0;
	while(temp > 0.0){
		loggamma += log(temp);
		temp -= 1.0;
	}
	p = temp = exp( (double)df / 2 * log(l / 2.0) - loggamma - l / 2.0);
	do{
		lastp = p;
		df += 2;
		temp *= l / (double)df;
		p += temp;
	} 
	while( fabs(p - lastp) > Eps);
	return p;
}

double prob_t(int df, double t)
/* ͳ٣Σʬۤ˽ΨѿϰϤȤΨ */
{
	int i;
	double cos2, p = 0.0, temp, ret;

	cos2 = 1.0 / ( 1.0 + (t * t)/df );
	i = ( df % 2 ) + 2;
	temp = sqrt(1.0 - cos2);
	while( i <= df){
		p += temp;
		temp *= ((double)i - 1.0 ) * cos2 / (double)i;
		i += 2;
	}
	ret = (df % 2) == 1 ? (p * sqrt(cos2) + atan(t / sqrt((double)df)))/Pi : p / 2.0;
	return ret;
}

double prob_f(int df1, int df2, double f)
/* ͳ(df1,df2)Σʬۤ˽ΨѿϰϤȤΨ */
{
	int i;
	double cos2, sin2, p, temp, ret;
	int flag;

	cos2 = 1.0 / ( 1.0 + (double)df1 * f / (double)df2);
	sin2 = 1.0 - cos2;
	if( ((df1 % 2) == 1) && ((df2 % 2) == 1)){
		p = atan(sqrt((double)df2 / ((double)df1 * f)));
		temp = sqrt(sin2 * cos2);
		i = 3;
		while( i <= df1 ){
			p += temp;
			temp *= (double)( i - 1 ) * sin2 / (double)i;
			i += 2;
		}
		temp *= (double)df1; 
		i = 3;
		while( i <= df2 ){
			p -= temp;
			temp *= (double)( df1 + i - 2 ) * cos2 / (double)i;
			i += 2;
		}
		ret = 1.0 - p * 2.0 / Pi;
	} 
	else {
		if( (df1 % 2) == 1){
			temp = cos2; 
			cos2 = sin2; 
			sin2 = temp;
			i = df1; 
			df1 = df2; 
			df2 = i; 
			flag = 1;
		} 
		else {
			flag = 0;
		}
		p = exp((double)df2 / 2.0 * log(cos2));
		temp = p; 
		i = 2;
		while(i < df1){
			temp *= (double)( df2 + i - 2 ) * sin2 / (double)i;
			p += temp; 
			i += 2;
		}
		ret = (flag == 1) ? p : 1 - p;
	}
	return ret;
}
